/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2001 CodeFactory AB
 * Copyright (C) 2001 Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Mikael Hallendal <micke@codefactory.se>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define d(x)

#include <bonobo.h>
#include "util/type-utils.h"
#include "allocation-manager-client.h"

static void  allocation_mc_init        (AllocationManagerClient    *rmc);
static void  allocation_mc_class_init  (GtkObjectClass             *klass);
static void  amc_destroy               (GtkObject                  *object);
static void  amc_listener              (BonoboListener             *listener,
					char                       *event_name,
					CORBA_any                  *any,
					CORBA_Environment          *ev,
					gpointer                    user_data);
static void  amc_project_set           (AllocationManagerClient    *amc);
static void  amc_project_unset         (AllocationManagerClient    *amc);

enum {
	ALLOCATION_ADDED,
	ALLOCATIONS_REMOVED,
	ALLOCATIONS_CLEAR,
	LAST_SIGNAL
};

static gint signals[LAST_SIGNAL] = { 0 };

struct _AllocationManagerClientPriv {
	GM_Shell                shell;
        GM_AllocationManager    manager;

	long                    listener_id;
	BonoboListener         *listener;
};

GNOME_CLASS_BOILERPLATE (AllocationManagerClient, allocation_mc,
                         GtkObject, gtk_object);

static void
allocation_mc_init (AllocationManagerClient *amc)
{
        AllocationManagerClientPriv   *priv;
        
	d(puts(__FUNCTION__));

        priv = g_new0 (AllocationManagerClientPriv, 1);
        
        amc->priv = priv;
}

static void
allocation_mc_class_init (GtkObjectClass *klass)
{
        klass->destroy = amc_destroy;

	d(puts(__FUNCTION__));

	signals[ALLOCATION_ADDED] = 
		gtk_signal_new ("allocation_added",
				GTK_RUN_LAST,
				klass->type,
				GTK_SIGNAL_OFFSET (AllocationManagerClientClass,
						   allocation_added),
				gtk_marshal_NONE__POINTER,
				GTK_TYPE_NONE,
				1, GTK_TYPE_POINTER);

	signals[ALLOCATIONS_REMOVED] = 
		gtk_signal_new ("allocations_removed",
				GTK_RUN_LAST,
				klass->type,
				GTK_SIGNAL_OFFSET (AllocationManagerClientClass,
						   allocations_removed),
				gtk_marshal_NONE__POINTER,
				GTK_TYPE_NONE,
				1, GTK_TYPE_POINTER);
	signals[ALLOCATIONS_CLEAR] =
		gtk_signal_new ("allocations_clear",
				GTK_RUN_LAST,
				klass->type,
				GTK_SIGNAL_OFFSET (AllocationManagerClientClass,
						   allocations_clear),
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);

	gtk_object_class_add_signals (klass, signals, LAST_SIGNAL);
}

static void
amc_destroy (GtkObject *object)
{
        AllocationManagerClient       *amc;
        AllocationManagerClientPriv   *priv;
	Bonobo_Listener                listener_co;
	Bonobo_EventSource             event_source_co;
	CORBA_Environment              ev;
	
        g_return_if_fail (object != NULL);
        g_return_if_fail (IS_ALLOCATION_MANAGER_CLIENT (object));
        
	d(puts (__FUNCTION__));

        amc  = ALLOCATION_MANAGER_CLIENT (object);
	priv = amc->priv;

	CORBA_exception_init (&ev);
	
	event_source_co = 
		Bonobo_Unknown_queryInterface (priv->shell,
					       "IDL:Bonobo/EventSource:1.0",
					       &ev);
	
	if (!BONOBO_EX (&ev) && event_source_co) {
		listener_co = 
			bonobo_object_corba_objref (BONOBO_OBJECT (priv->listener));
		Bonobo_EventSource_removeListener (event_source_co,
						   priv->listener_id,
						   &ev);
		bonobo_object_release_unref (event_source_co, NULL);
	}

	bonobo_object_unref (BONOBO_OBJECT (priv->listener));
	
	CORBA_Object_release (priv->shell, &ev);
	CORBA_exception_free (&ev);
	
	g_free (amc->priv);
        amc->priv = NULL;
}

static void 
amc_listener (BonoboListener      *listener,
	      char                *event_name,
	      CORBA_any           *any,
	      CORBA_Environment   *ev,
	      gpointer             user_data)
{
	AllocationManagerClient       *amc;
	AllocationManagerClientPriv   *priv;
        
	g_return_if_fail (user_data != NULL);
	g_return_if_fail (IS_ALLOCATION_MANAGER_CLIENT (user_data));
	
	d(puts(__FUNCTION__));

	amc  = ALLOCATION_MANAGER_CLIENT (user_data);
	priv = amc->priv;
	
	/* Emit signals */

	if (strstr (event_name, "allocation:added")) {
                GM_Allocation   *allocation;

                allocation = (GM_Allocation *) any->_value;
                
                gtk_signal_emit (GTK_OBJECT (amc),
                                 signals[ALLOCATION_ADDED],
                                 allocation);
	}
	else if (strstr (event_name, "allocation:removed")) {
                GM_AllocationSeq   *allocations;
                GSList             *list;
                
                allocations = (GM_AllocationSeq *) any->_value;

		if (allocations) {
			list = corba_util_allocation_seq_to_list (allocations);
			
			gtk_signal_emit (GTK_OBJECT (amc),
					 signals[ALLOCATIONS_REMOVED],
					 list);

			g_slist_CORBA_free (list);
		} else {
			g_warning ("Got empty deassignment");
		}
	}
	else if (strstr (event_name, "project:set")) {
 		amc_project_set (amc);
	}
 	else if (strstr (event_name, "project:unset")) {
		gtk_signal_emit (GTK_OBJECT (amc),
				 signals[ALLOCATIONS_CLEAR]);
		
 		amc_project_unset (amc);
	} else {
		g_print ("AllocationManagerClient: got unhandled event: %s\n",
		 	 event_name);
	}
}

static void
amc_project_set (AllocationManagerClient *amc) 
{
	AllocationManagerClientPriv   *priv;
	GM_Project                     project_co;
	CORBA_Environment              ev;
	
	d(puts (__FUNCTION__));

	priv = amc->priv;

	if (priv->manager) {
		bonobo_object_release_unref (priv->manager, NULL);
	}

	if (!priv->shell) {
		g_warning ("AllocationManagerClient doesn't have a shell");
		return;
	}

	CORBA_exception_init (&ev);

	project_co = GNOME_MrProject_Shell_getProject (priv->shell, &ev);
	
	if (BONOBO_EX (&ev) || !project_co) {
		g_warning ("Shell doesn't have a project");
		CORBA_exception_free (&ev);
		return;
	}

	priv->manager = Bonobo_Unknown_queryInterface (project_co,
						       "IDL:GNOME/MrProject/AllocationManager:1.0",
						       &ev);
	
	if (BONOBO_EX (&ev)) {
		g_warning ("Couldn't get AllocationManager interface through project");
	}
	
	CORBA_exception_free (&ev);
}

static void
amc_project_unset (AllocationManagerClient *amc)
{
	AllocationManagerClientPriv   *priv;

	d(puts (__FUNCTION__));

	priv = amc->priv;

	if (priv->manager) {
		bonobo_object_release_unref (priv->manager, NULL);
	}

	priv->manager = CORBA_OBJECT_NIL;
}

AllocationManagerClient *
allocation_mc_new (GM_Shell shell, gboolean set, CORBA_Environment *ev)
{
	AllocationManagerClient       *amc;
	AllocationManagerClientPriv   *priv;
	Bonobo_EventSource             event_source_co;
	Bonobo_Listener                listener_co;
	
	d(puts (__FUNCTION__));

	amc  = gtk_type_new (ALLOCATION_MANAGER_CLIENT_TYPE);
	priv = amc->priv;
	
	priv->shell = CORBA_Object_duplicate (shell, ev);

	if (BONOBO_EX (ev) || !priv->shell) {
		g_warning ("Couldn't duplicate shell\n");
		return NULL;
	}

	event_source_co = 
		Bonobo_Unknown_queryInterface (priv->shell,
					       "IDL:Bonobo/EventSource:1.0",
					       ev);
	
	if (BONOBO_EX (ev) || !event_source_co) {
		g_warning ("Couldn't get shell event source");
		return NULL;
	}
	
	priv->listener = bonobo_listener_new (amc_listener, amc);
	
	listener_co = BONOBO_OBJREF (priv->listener);

	priv->listener_id = 
		Bonobo_EventSource_addListenerWithMask (
			event_source_co,
			listener_co,
			"GNOME/MrProject:allocation,"
			"GNOME/MrProject:project",
			ev);
	
	if (BONOBO_EX (ev)) {
		g_warning ("Couldn't add listener");
		bonobo_object_unref (BONOBO_OBJECT (priv->listener));
	}

	bonobo_object_release_unref (event_source_co, NULL);

	if (set) {
		amc_project_set (amc);
	}
 	
	return amc;
}

void
allocation_mc_allocate (AllocationManagerClient   *amc,
			GM_Id                      task_id,
			GM_Id                      resource_id,
			short                      units,
			CORBA_Environment         *ev)
{
	AllocationManagerClientPriv   *priv;

	g_return_if_fail (amc != NULL);
	g_return_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc));

	d(puts(__FUNCTION__));

	priv = amc->priv;

	GNOME_MrProject_AllocationManager_allocate (priv->manager,
						    task_id,
						    resource_id,
						    ev);

}

void
allocation_mc_deallocate (AllocationManagerClient   *amc,
			  GM_Allocation             *allocation,
			  CORBA_Environment        *ev)
{
	AllocationManagerClientPriv   *priv;
	
	g_return_if_fail (amc != NULL);
	g_return_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc));

	d(puts(__FUNCTION__));

	priv = amc->priv;
	
	GNOME_MrProject_AllocationManager_deallocate (priv->manager,
						      allocation,
						      ev);
	
}

void
allocation_mc_deallocate_ids (AllocationManagerClient   *amc,
			      GM_Id                      task_id,
			      GM_Id                      resource_id,
			      CORBA_Environment         *ev)
{
	GM_Allocation   *allocation;

	allocation             = GNOME_MrProject_Allocation__alloc ();
	allocation->taskId     = task_id;
	allocation->resourceId = resource_id;
	allocation->units      = 0;
	
	d(puts(__FUNCTION__));

	allocation_mc_deallocate (amc, allocation, ev);
	
	CORBA_free (allocation);
}

GSList *
allocation_mc_get_allocations_by_task (AllocationManagerClient   *amc,
				       GM_Id                      task_id,
				       CORBA_Environment         *ev)
{
	AllocationManagerClientPriv   *priv;
	GM_AllocationSeq              *allocations;
	GSList                        *list;

	g_return_val_if_fail (amc != NULL, NULL);
	g_return_val_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc), NULL);

	d(puts(__FUNCTION__));

	priv = amc->priv;
	
	allocations =  GNOME_MrProject_AllocationManager_getAllocationsByTask (priv->manager,
									       task_id,
									       ev);
	
	/* FIX: Check for exceptions */

	if (BONOBO_EX (ev) || !allocations) {
		g_warning ("Couldn't get initial allocations");
		return NULL;
	}
	

	list = corba_util_allocation_seq_to_list (allocations);

	CORBA_free (allocations);

	return list;
}

GSList *       
allocation_mc_get_allocations_by_resource (AllocationManagerClient   *amc,
					   GM_Id                      res_id,
					   CORBA_Environment         *ev)
{
	AllocationManagerClientPriv   *priv;
	GM_AllocationSeq              *allocations;
	GSList                        *list;
	
	g_return_val_if_fail (amc != NULL, NULL);
	g_return_val_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc), NULL);

	d(puts(__FUNCTION__));

	priv = amc->priv;

	allocations = GNOME_MrProject_AllocationManager_getAllocationsByResource (priv->manager,
										  res_id,
										  ev);
	
	/* FIX: Check for exceptions */

	list = corba_util_allocation_seq_to_list (allocations);

	CORBA_free (allocations);
	
	return list;
}

GSList *           
allocation_mc_get_all_allocations (AllocationManagerClient   *amc,
				   CORBA_Environment        *ev)
{
	AllocationManagerClientPriv   *priv;
	GM_AllocationSeq              *allocations;
	GSList                        *list;
	
	g_return_val_if_fail (amc != NULL, NULL);
	g_return_val_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc), NULL);

	d(puts(__FUNCTION__));

	priv = amc->priv;

	allocations = GNOME_MrProject_AllocationManager_getAllAllocations (priv->manager,
									ev);
	
	/* FIX: Check for exceptions */
	list = corba_util_allocation_seq_to_list (allocations);

	CORBA_free (allocations);
	
	return list;
}

void
allocation_mc_deallocate_all_resources (AllocationManagerClient   *amc,
					GM_Id                      task_id,
					CORBA_Environment        *ev)
{
	AllocationManagerClientPriv   *priv;
	
	g_return_if_fail (amc != NULL);
	g_return_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc));

	d(puts(__FUNCTION__));

	priv = amc->priv;

	GNOME_MrProject_AllocationManager_deallocateAllResources(priv->manager,
								 task_id,
								 ev);

}

void
allocation_mc_deallocate_all_tasks (AllocationManagerClient   *amc,
				    GM_Id                      resource_id,
				    CORBA_Environment        *ev)
{
	AllocationManagerClientPriv   *priv;
	
	g_return_if_fail (amc != NULL);
	g_return_if_fail (IS_ALLOCATION_MANAGER_CLIENT(amc));

	d(puts(__FUNCTION__));

	priv = amc->priv;

	GNOME_MrProject_AllocationManager_deallocateAllTasks (priv->manager,
							      resource_id,
							      ev);
	
}



