/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2001 CodeFactory AB
 * Copyright (C) 2001 Richard Hult <rhult@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/gnome-preferences.h>
#include <libgnomeui/gnome-dialog.h>
#include <libgnomeui/gnome-stock.h>
#include <bonobo/bonobo-control.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gdk-pixbuf/gdk-pixbuf-loader.h>
#include <gal/widgets/gtk-combo-box.h>
#include "util/time-utils.h"
#include "gantt-goto-control.h"

static GtkWidget *image_from_file (const gchar *filename);

enum {
	DIALOG_SHOWN,
	DIALOG_HIDDEN,
	DATE_CLICKED,
	TODAY_CLICKED,
	SELECTED_CLICKED,
	START_CLICKED,
	LAST_SIGNAL
};

typedef struct {
	GtkWidget *today;
	GtkWidget *selected;
	GtkWidget *start;
	GtkWidget *calendar;
} GotoButtons;

static gint signals [LAST_SIGNAL] = { 0, };

static void
gantt_goto_control_destroy (GtkObject *object)
{
        gtk_widget_destroy (GANTT_GOTO_CONTROL (object)->combo);
}

static void
gantt_goto_control_init (GanttGotoControl *control)
{
}

static void
gantt_goto_control_class_init (GanttGotoControlClass *klass)
{
        GtkObjectClass *object_class;

        object_class = (GtkObjectClass *) klass;

        object_class->destroy = gantt_goto_control_destroy;

	signals [DIALOG_SHOWN] =
		gtk_signal_new ("dialog-shown",
				GTK_RUN_LAST,
				object_class->type,
				0,
				gtk_marshal_BOOL__NONE,
				GTK_TYPE_BOOL, 0);

	signals [DIALOG_HIDDEN] =
		gtk_signal_new ("dialog-hidden",
				GTK_RUN_LAST,
				object_class->type,
				0,
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);

	signals [DATE_CLICKED] =
		gtk_signal_new ("date-clicked",
				GTK_RUN_LAST,
				object_class->type,
				0,
				gtk_marshal_NONE__INT,
				GTK_TYPE_NONE, 1, GTK_TYPE_LONG);

	signals [TODAY_CLICKED] =
		gtk_signal_new ("today-clicked",
				GTK_RUN_LAST,
				object_class->type,
				0,
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);

	signals [SELECTED_CLICKED] =
		gtk_signal_new ("selected-clicked",
				GTK_RUN_LAST,
				object_class->type,
				0,
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);

	signals [START_CLICKED] =
		gtk_signal_new ("start-clicked",
				GTK_RUN_LAST,
				object_class->type,
				0,
				gtk_marshal_NONE__NONE,
				GTK_TYPE_NONE, 0);
	
	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
}

GtkType
gantt_goto_control_get_type (void)
{
        static GtkType goto_type = 0;

        if (!goto_type)
        {
                const GtkTypeInfo goto_info =
                {
                        "GanttGotoControl",
                        sizeof (GanttGotoControl),
                        sizeof (GanttGotoControlClass),
                        (GtkClassInitFunc)  gantt_goto_control_class_init,
                        (GtkObjectInitFunc) gantt_goto_control_init,
                        NULL, /* Reserved 1 */
                        NULL, /* Reserved 2 */
                        (GtkClassInitFunc) NULL,
                };

                goto_type = gtk_type_unique (bonobo_control_get_type (),
                                             &goto_info);
        }

        return goto_type;
}

static void
month_changed (GtkWidget *widget, GanttGotoControl *control)
{
	/*g_print ("month changed\n");*/
}

static void
date_selected (GtkWidget *widget, GanttGotoControl *control)
{
	time_t t;
	guint  year, month, day;

	g_print ("month changed\n");
		
	gtk_calendar_get_date (GTK_CALENDAR (control->calendar),
			       &year, &month, &day);

	t = time_from_day (year, month, day);
	
	gtk_combo_box_popup_hide (GTK_COMBO_BOX (control->combo));
	gtk_signal_emit (GTK_OBJECT (control), signals[DATE_CLICKED], t);
}

static void
today_clicked (GtkWidget *widget, GanttGotoControl *control)
{
	gtk_combo_box_popup_hide (GTK_COMBO_BOX (control->combo));
	gtk_signal_emit (GTK_OBJECT (control), signals[TODAY_CLICKED], NULL);
}

static void
start_clicked (GtkWidget *widget, GanttGotoControl *control)
{
	gtk_combo_box_popup_hide (GTK_COMBO_BOX (control->combo));
	gtk_signal_emit (GTK_OBJECT (control), signals[START_CLICKED], NULL);
}

static void
selected_clicked (GtkWidget *widget, GanttGotoControl *control)
{
	gtk_combo_box_popup_hide (GTK_COMBO_BOX (control->combo));
	gtk_signal_emit (GTK_OBJECT (control), signals[SELECTED_CLICKED], NULL);
}

static GtkWidget *
create_goto_frame (GotoButtons *buttons, gboolean is_dialog)
{
	GtkWidget *frame, *calendar;
	GtkWidget *vbox, *button_box, *button;

	frame = gtk_frame_new (NULL);
	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);

	vbox = gtk_vbox_new (FALSE, 0);
	gtk_container_add (GTK_CONTAINER (frame), vbox);
	gtk_widget_show (vbox);
	
	calendar = gtk_calendar_new ();
	buttons->calendar = calendar;
	gtk_box_pack_start (GTK_BOX (vbox), calendar, FALSE, FALSE, 0);
	gtk_widget_show (calendar);

	if (TRUE /*is_dialog*/) {
		GtkWidget *label, *alignment;
		
		label = gtk_label_new (_("Double click on a date to go there"));
		alignment = gtk_alignment_new (1, 0, 0, 0);
		gtk_container_add (GTK_CONTAINER (alignment), label);
		gtk_widget_show_all (alignment);
		gtk_box_pack_start (GTK_BOX (vbox), alignment, FALSE, FALSE, 4);
	}

	button_box = gtk_hbutton_box_new ();
	gtk_container_set_border_width (GTK_CONTAINER (button_box), 4);
	gtk_button_box_set_spacing (GTK_BUTTON_BOX (button_box), 2);
	gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (button_box), 2, 0);
	gtk_button_box_set_child_size (GTK_BUTTON_BOX (button_box), 0, 0);
	gtk_widget_show (button_box);

	gtk_box_pack_start (GTK_BOX (vbox), button_box, FALSE, FALSE, 0);

	button = gtk_button_new_with_label (_("Today"));
	buttons->today = button;
	gtk_widget_show (button);
	gtk_box_pack_start (GTK_BOX (button_box), button, FALSE, FALSE, 0);
	
	button = gtk_button_new_with_label (_("Project start"));
	buttons->start = button;
	gtk_widget_show (button);
	gtk_box_pack_start (GTK_BOX (button_box), button, FALSE, FALSE, 0);
	
	button = gtk_button_new_with_label (_("Selected task"));
	buttons->selected = button;
	gtk_widget_show (button);
	gtk_box_pack_start (GTK_BOX (button_box), button, FALSE, FALSE, 0);
	
	return frame;
}

static gboolean
goto_dialog_close (GtkWidget *widget, GanttGotoControl *control)
{
	gtk_signal_emit (GTK_OBJECT (control),
			 signals[DIALOG_HIDDEN],
			 NULL);

	return FALSE; /* Yes, close the window. */
}

static void
goto_dialog_button_clicked (GtkWidget        *widget,
			    gint              btn,
			    GanttGotoControl *control)
{
	gnome_dialog_close (GNOME_DIALOG (widget));
}

static void
goto_button_clicked (GtkWidget *widget, GanttGotoControl *control)
{
	GtkWidget   *dialog, *frame;
	GotoButtons  buttons;
	gboolean     ok_to_show;

	/* First, see if it's ok to show the dialog. E.g. if there
	 * already is one shown, we shouldn't show one more.
	 */
	gtk_signal_emit (GTK_OBJECT (control),
			 signals[DIALOG_SHOWN],
			 &ok_to_show);
	
	if (!ok_to_show) {
		return;
	}

	dialog = gnome_dialog_new (_("Go to"),
				   GNOME_STOCK_BUTTON_CLOSE,
				   NULL);

	frame = create_goto_frame (&buttons, TRUE); /* is dialog */
	gtk_widget_show (frame);
	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
			    frame, FALSE, FALSE, 0);
	
	control->calendar = buttons.calendar;
	gtk_signal_connect (GTK_OBJECT (buttons.calendar),
			    "day-selected-double-click",
			    date_selected,
			    control);
	
	gtk_signal_connect (GTK_OBJECT (buttons.today),
			    "clicked",
			    GTK_SIGNAL_FUNC (today_clicked),
			    control);
	gtk_signal_connect (GTK_OBJECT (buttons.selected),
			    "clicked",
			    GTK_SIGNAL_FUNC (selected_clicked),
			    control);
	gtk_signal_connect (GTK_OBJECT (buttons.start),
			    "clicked",
			    GTK_SIGNAL_FUNC (start_clicked),
			    control);

	gtk_signal_connect (GTK_OBJECT (dialog),
			    "close",
			    GTK_SIGNAL_FUNC (goto_dialog_close),
			    control);

	gtk_signal_connect (GTK_OBJECT (dialog),
			    "clicked",
			    goto_dialog_button_clicked,
			    control);

	gtk_widget_show (dialog);
}

BonoboControl *
gantt_goto_control_new (void)
{
	GanttGotoControl *control;
	GotoButtons       buttons;
	GtkWidget        *combo, *image, *frame;
	GtkWidget        *button;

        control = gtk_type_new (gantt_goto_control_get_type ());

	image = image_from_file (MRPROJECT_IMAGEDIR "24_calendar.png");
	gtk_widget_show (image);

	button = gtk_button_new ();
	if (!gnome_preferences_get_toolbar_relief_btn ()) {
		gtk_button_set_relief (GTK_BUTTON (button),
				       GTK_RELIEF_NONE);
	}
	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
	gtk_container_add (GTK_CONTAINER (button), image);
	gtk_widget_show (button);

	gtk_signal_connect (GTK_OBJECT (button),
			    "clicked",
			    goto_button_clicked,
			    control);
	
	frame = create_goto_frame (&buttons, FALSE); /* is not dialog */
	control->calendar = buttons.calendar;
	gtk_signal_connect (GTK_OBJECT (buttons.calendar),
			    "next-month",
			    month_changed,
			    control);
	gtk_signal_connect (GTK_OBJECT (buttons.calendar),
			    "day-selected-double-click",
			    date_selected,
			    control);

	gtk_signal_connect (GTK_OBJECT (buttons.today),
			    "clicked",
			    GTK_SIGNAL_FUNC (today_clicked),
			    control);
	gtk_signal_connect (GTK_OBJECT (buttons.selected),
			    "clicked",
			    GTK_SIGNAL_FUNC (selected_clicked),
			    control);
	gtk_signal_connect (GTK_OBJECT (buttons.start),
			    "clicked",
			    GTK_SIGNAL_FUNC (start_clicked),
			    control);

	combo = gtk_combo_box_new (button, frame);
	control->combo = combo;

	if (!gnome_preferences_get_toolbar_relief_btn ()) {
		gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (combo),
						GTK_RELIEF_NONE);
	}

	/* We don't want it to be tearable since we have our own
	 * toplevel dialog for this.
	 */
	gtk_combo_box_set_tearable (GTK_COMBO_BOX (combo), FALSE);

	gtk_combo_box_set_title (GTK_COMBO_BOX (combo), _("Go to"));
	gtk_container_set_border_width (GTK_CONTAINER (combo), 0);
	
        gtk_widget_show_all (combo);
	return bonobo_control_construct (BONOBO_CONTROL (control), combo);
}

static GtkWidget *
image_from_file (const gchar *filename)
{
	GtkWidget *image;
	GdkPixmap *pixmap, *bitmap;
	GdkPixbuf *pixbuf;

	pixbuf = gdk_pixbuf_new_from_file (filename);
	gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, &bitmap, 127);
	image = gtk_pixmap_new (pixmap, bitmap);

	gdk_pixmap_unref (pixmap);
	gdk_pixmap_unref (bitmap);
	gdk_pixbuf_unref (pixbuf);
	
	return image;
}

