/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2001 CodeFactory AB
 * Copyright (C) 2001 Richard Hult <rhult@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <math.h>
#include <string.h>
#include <libgnomeui/gnome-canvas.h>
#include <libgnomeui/gnome-canvas-util.h>
#include "util/type-utils.h"
#include "gantt-model.h"
#include "gantt-scale.h"
#include "gantt-link-item.h"
#include "gantt-row-item.h"

/* Arguments. */
enum {
	ARG_0,
	ARG_ROW1,
	ARG_ROW2,
	ARG_X,
	ARG_Y
};

typedef enum {
	LINK_FS
} LinkType;

/* Private members. */
struct _GanttLinkItemPriv {
	GnomeCanvasItem      *row1;
	GnomeCanvasItem      *row2;
	gdouble		      x1, y1; /* row 1 */
	gdouble		      x2, y2; /* row 2 */
	LinkType              type;
};

/* GtkObject. */
static void gantt_link_item_class_init (GanttLinkItemClass *klass);
static void gantt_link_item_init       (GanttLinkItem      *item);

static GdkGC *frame_gc, *text_gc;

GNOME_CLASS_BOILERPLATE (GanttLinkItem,
			 gantt_link_item,
			 GnomeCanvasItem,
			 gnome_canvas_item);

static void
gantt_link_item_destroy (GtkObject *object)
{
	GanttLinkItem *gantt;

	g_return_if_fail (object != NULL);
	g_return_if_fail (IS_GANTT_LINK_ITEM (object));

	gantt = GANTT_LINK_ITEM (object);

	g_free (gantt->priv);
	gantt->priv = NULL;

	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS,
				   destroy,
				   (object));
}

static void
row1_changed (GanttRowItem  *row, 
	      gdouble        x1, 
	      gdouble        x2,
	      gdouble        y1,
	      gdouble        y2,
	      GanttLinkItem *link)
{
	switch (link->priv->type) {
	case LINK_FS:
		link->priv->x1 = x2;
		link->priv->y1 = y1 + (y2 - y1) / 2;
		break;
	}

	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (link));
}

static void
row1_destroyed (GanttRowItem *row, GanttLinkItem *link)
{
	gtk_object_destroy (GTK_OBJECT (link));
}

static void
row2_changed (GanttRowItem  *row,
	      gdouble        x1, 
	      gdouble        x2,
	      gdouble        y1,
	      gdouble        y2,
	      GanttLinkItem *link)
{
	switch (link->priv->type) {
	case LINK_FS:
		link->priv->x2 = x1;
		if (y1 > link->priv->y1) {
			/* This row is below the other one. */
			link->priv->y2 = y1;
		} else {
			link->priv->y2 = y2;
		}
		break;
	}

	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (link));
}

static void
row2_destroyed (GanttRowItem *row, GanttLinkItem *link)
{
	gtk_object_destroy (GTK_OBJECT (link));
}

static void
gantt_link_item_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
{
	GanttLinkItem     *link;
	GanttLinkItemPriv *priv;
	gdouble            x, y, width, height;
	
	link = GANTT_LINK_ITEM (object);
	priv = link->priv;

	switch (arg_id) {
	case ARG_ROW1:
		priv->row1 = GNOME_CANVAS_ITEM (GTK_VALUE_OBJECT (*arg));
		gtk_object_get (GTK_OBJECT (priv->row1), 
				"x", &x,
				"y", &y,
				"width", &width,
				"height", &height,
				NULL);
		row1_changed (GANTT_ROW_ITEM (priv->row1), 
			      x, x + width, 
			      y, y + height, 
			      link);
		gtk_signal_connect_while_alive (GTK_OBJECT (priv->row1),
						"changed",
						row1_changed,
						object, object);
		gtk_signal_connect_while_alive (GTK_OBJECT (priv->row1),
						"destroy",
						row1_destroyed,
						object, object);
		break;
		
	case ARG_ROW2:
		priv->row2 = GNOME_CANVAS_ITEM (GTK_VALUE_OBJECT (*arg));
		gtk_object_get (GTK_OBJECT (priv->row2), 
				"x", &x,
				"y", &y,
				"width", &width,
				"height", &height,
				NULL);
		row2_changed (GANTT_ROW_ITEM (priv->row2),
			      x, x + width, 
			      y, y + height, 
			      link);
		gtk_signal_connect_while_alive (GTK_OBJECT (priv->row2),
						"changed",
						row2_changed,
						object, object);
		gtk_signal_connect_while_alive (GTK_OBJECT (priv->row2),
				    "destroy",
				    row2_destroyed,
				    object, object);
		break;

	default:
		return;
	}

	gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (link));
}

static void
gantt_link_item_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
{
	GanttLinkItem     *link;
	GanttLinkItemPriv *priv;
	
	link = GANTT_LINK_ITEM (object);
	priv = link->priv;

	switch (arg_id) {
	case ARG_ROW1:
		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (priv->row1);
		break;

	case ARG_ROW2:
		GTK_VALUE_OBJECT (*arg) = GTK_OBJECT (priv->row2);
		break;

	default:
		arg->type = GTK_TYPE_INVALID;
		break;
	}
}

static void
gantt_link_item_realize (GnomeCanvasItem *item)
{
	GtkStyle      *style;
	/*GanttLinkItem *link = GANTT_LINK_ITEM (item);*/

	GNOME_CALL_PARENT_HANDLER (GNOME_CANVAS_ITEM_CLASS,
				   realize,
				   (item));

	style = GTK_WIDGET (item->canvas)->style;

	if (!text_gc) {
		text_gc = gdk_gc_new (GTK_WIDGET (item->canvas)->window);
		gdk_gc_set_foreground (text_gc,
				       &style->fg[GTK_STATE_NORMAL]);
	}

	if (!frame_gc) {
		frame_gc = gdk_gc_new (GTK_WIDGET (item->canvas)->window);
		gdk_gc_set_foreground (frame_gc,
				       &style->fg[GTK_STATE_NORMAL]);
	}
}

static void
gantt_link_item_unrealize (GnomeCanvasItem *item)
{
	/*GanttLinkItem *link = GANTT_LINK_ITEM (item);*/

	GNOME_CALL_PARENT_HANDLER (GNOME_CANVAS_ITEM_CLASS,
				   unrealize,
				   (item));
}

static int
gantt_link_item_event (GnomeCanvasItem *item, GdkEvent *event)
{
	/* No events for now. */
	return FALSE;
}

static double 
gantt_link_item_point (GnomeCanvasItem  *item,
		       double            x,
		       double            y,
		       int               cx,
		       int               cy,
		       GnomeCanvasItem **actual_item)
{
	/**actual_item = item;*/
	return 1.0;
}

static void
gantt_link_item_bounds_item_coordinates (GnomeCanvasItem *item,
					 double          *x1,
					 double          *y1,
					 double          *x2,
					 double          *y2)
{
	GanttLinkItem     *link;
	GanttLinkItemPriv *priv;

	link = GANTT_LINK_ITEM (item);
	priv = link->priv;

	*x1 = MIN (priv->x1, priv->x2) - 4;
	*y1 = MIN (priv->y1, priv->y2) - 1;
	*x2 = MAX (priv->x1, priv->x2) + 6;
	*y2 = MAX (priv->y1, priv->y2) + 2;
}

static void
gantt_link_item_bounds_canvas_coordinates (GnomeCanvasItem *item,
					   double          *x1,
					   double          *y1,
					   double          *x2,
					   double          *y2)
{
	double             i2c [6];
	ArtPoint           c1, c2, i1, i2;
	GanttLinkItem     *link;
	GanttLinkItemPriv *priv;

	link = GANTT_LINK_ITEM (item);
	priv = link->priv;

	gantt_link_item_bounds_item_coordinates (item, x1, y1, x2, y2);

	gnome_canvas_item_i2c_affine (item, i2c);
	
	i1.x = *x1;
	i1.y = *y1;
	i2.x = *x2;
	i2.y = *y2;
	art_affine_point (&c1, &i1, i2c);
	art_affine_point (&c2, &i2, i2c);
	*x1 = c1.x;
	*y1 = c1.y;
	*x2 = c2.x;
	*y2 = c2.y;
}

static void
gantt_link_item_update (GnomeCanvasItem *item,
			double          *affine,
			ArtSVP          *clip_path,
			int              flags)
{
	GanttLinkItem     *link;
	GanttLinkItemPriv *priv;
	gdouble            x1, y1, x2, y2;

	link = GANTT_LINK_ITEM (item);
	priv = link->priv;

	GNOME_CALL_PARENT_HANDLER (GNOME_CANVAS_ITEM_CLASS,
				   update,
				   (item, affine, clip_path,
				    flags));

	gantt_link_item_bounds_canvas_coordinates (item, &x1, &y1, &x2, &y2);
	gnome_canvas_update_bbox (item, x1, y1, x2, y2);
}

static void 
gantt_link_item_draw (GnomeCanvasItem *item,
		      GdkDrawable     *drawable,
		      int              x,
		      int              y,
		      int              width,
		      int              height)
{
	GanttLinkItem     *link;
	GanttLinkItemPriv *priv;
	double             i2c[6];
	gint               cx1, cy1, cx2, cy2;
	ArtPoint           pc, pi;
	gint               direction;

	link = GANTT_LINK_ITEM (item);
	priv = link->priv;

	gnome_canvas_item_i2c_affine (item, i2c);

	pi.x = priv->x1;
	pi.y = priv->y1;
	art_affine_point (&pc, &pi, i2c);
	cx1 = floor (pc.x + 0.5) - x;
	cy1 = floor (pc.y + 0.5) - y;
	
	pi.x = priv->x2;
	pi.y = priv->y2;
	art_affine_point (&pc, &pi, i2c);
	cx2 = floor (pc.x + 0.5) - x;
	cy2 = floor (pc.y + 0.5) - y; 
	
	if (cy1 > cy2) { 
		direction = -1;
	} else { 
		direction = 1;
	}

	gdk_draw_line (drawable,
		       frame_gc,
		       cx1, cy1,
		       cx2, cy1);
	gdk_draw_line (drawable,
		       frame_gc,
		       cx2, cy1,
		       cx2, cy2); 

	gdk_draw_line (drawable,
		       frame_gc,
		       cx2, cy2,
		       cx2 - 4, cy2 - 6 * direction);
	gdk_draw_line (drawable,
		       frame_gc,
		       cx2, cy2,
		       cx2 + 4, cy2 - 6 * direction);
}

static void
gantt_link_item_class_init (GanttLinkItemClass *klass)
{
	GtkObjectClass *object_class;
	GnomeCanvasItemClass *item_class;
	
	object_class = (GtkObjectClass*) klass;
	item_class = (GnomeCanvasItemClass*) klass;

	/* Gtk Object methods. */
	object_class->destroy = gantt_link_item_destroy;
	object_class->set_arg = gantt_link_item_set_arg;
	object_class->get_arg = gantt_link_item_get_arg;

	/* GnomeCanvasItem methods. */
	item_class->update = gantt_link_item_update;
	item_class->draw = gantt_link_item_draw;
	item_class->point = gantt_link_item_point;
	item_class->bounds = gantt_link_item_bounds_item_coordinates;
	item_class->event = gantt_link_item_event;
	item_class->realize = gantt_link_item_realize;
	item_class->unrealize = gantt_link_item_unrealize;

	/* Arguments. */
	gtk_object_add_arg_type ("GanttLinkItem::row1",
				 GTK_TYPE_OBJECT,
				 GTK_ARG_READWRITE,
				 ARG_ROW1);
	gtk_object_add_arg_type ("GanttLinkItem::row2",
				 GTK_TYPE_OBJECT,
				 GTK_ARG_READWRITE,
				 ARG_ROW2);
}

static void
gantt_link_item_init (GanttLinkItem *item)
{
	item->priv = g_new0 (GanttLinkItemPriv, 1);
	item->priv->x1 = 0;
	item->priv->y1 = 0;
	item->priv->x2 = 0;
	item->priv->y2 = 0;

	item->priv->type = LINK_FS;
}


