/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2001 CodeFactory AB
 * Copyright (C) 2001 Richard Hult <rhult@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <bonobo.h>
#include <liboaf/liboaf.h>
#include <orb/orbit.h>
#include <gtk/gtkobject.h>
#include <gnome-xml/parser.h>
#include <gnome-xml/parserInternals.h>
#include <gnome-xml/xmlmemory.h>
#include "util/string-utils.h"
#include "util/type-utils.h"
#include "file-task-manager.h"
#include "file-resource-manager.h"
#include "file-allocation-manager.h"
#include "file-project.h"


static void file_project_init       (FileProject      *project);
static void file_project_class_init (FileProjectClass *klass);

GNOME_CLASS_BOILERPLATE (FileProject, file_project,
			 Project, project);


static void
fp_load (Project           *project,
	 const gchar       *method,
	 const gchar       *location,
	 CORBA_Environment *ev)
{
	CORBA_Environment      e;
	CORBA_Object           filter_co;
	const gchar           *query;
	OAF_ServerInfoList    *oaf_result;
	guint                  i;
	OAF_ServerInfo         server;
	gboolean               format_ok, loaded_ok;
	
	CORBA_exception_init (&e);
	
	/* Query for all components that can load a project file. */
	query = "repo_ids.has ('IDL:GNOME/MrProject/FileFilter:1.0')";
	
	oaf_result = oaf_query (query, NULL, &e);

	loaded_ok = FALSE;
	if (!BONOBO_EX (&e) && oaf_result != NULL) {
		for (i = 0; i < oaf_result->_length; i++) {
			server = oaf_result->_buffer[i];

			filter_co = oaf_activate_from_id (server.iid,
							  0,	/* OAF_ActivationFlags */
							  NULL, /* OAF_ActivationID* */
							  &e);
			
			if (BONOBO_EX (&e) || !filter_co) {
				CORBA_exception_free (&e);
				continue;
			}

			format_ok = GNOME_MrProject_FileFilter_probe (
				filter_co,
				method,
				location,
				&e);
			
			if (BONOBO_EX (&e)) {
				CORBA_exception_free (&e);
				continue;
			}

			if (format_ok) {
				/*GNOME_MrProject_FileSummary *summary;

				summary = GNOME_MrProject_FileSummary__alloc ();

				GNOME_MrProject_FileFilter_loadSummary (
					filter_co,
					method,
					location,
					&summary,
					&e);

					CORBA_free (summary);*/

				GNOME_MrProject_FileFilter_load (
					filter_co,
					method,
					location,
					BONOBO_OBJREF (project),
					&e);

				if (BONOBO_EX (&e)) {
					/* Ok, try the next one. */
					CORBA_exception_free (&e);
					continue;
				} else {
					loaded_ok = TRUE;
					break;
				}
			}
		}
		
		if (oaf_result) {
			CORBA_free (oaf_result);
		}
		
		CORBA_exception_free (&e);
	}

	if (!loaded_ok) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_Project_IOFailure,
				     NULL);
	}
}

static void
fp_save (Project           *project,
	 const gchar       *method,
	 const gchar       *location,
	 CORBA_Environment *ev)
{
	CORBA_Environment      e;
	CORBA_Object           filter_co;
	const gchar           *query;
	OAF_ServerInfoList    *oaf_result;
	guint                  i;
	OAF_ServerInfo         server;
	gboolean               saved_ok;
	
	CORBA_exception_init (&e);
	
	/* Query for all components that can load a project file. */
	query = "repo_ids.has ('IDL:GNOME/MrProject/FileFilter:1.0')";
	
	oaf_result = oaf_query (query, NULL, &e);
		
	saved_ok = FALSE;
	if (!BONOBO_EX (&e) && oaf_result != NULL) {
		for (i = 0; i < oaf_result->_length; i++) {
			server = oaf_result->_buffer[i];

			filter_co = oaf_activate_from_id (server.iid,
							  0,	/* OAF_ActivationFlags */
							  NULL, /* OAF_ActivationID* */
							  &e);
			
			if (BONOBO_EX (&e) || !filter_co) {
				CORBA_exception_free (&e);
				continue;
			}

			GNOME_MrProject_FileFilter_save (
				filter_co,
				method,
				location,
				BONOBO_OBJREF (project),
				&e);
			
			if (BONOBO_EX (&e)) {
				/* Ok, try the next one. */
				CORBA_exception_free (&e);
				continue;
			} else {
				saved_ok = TRUE;
				break;
			}
		}
		
		if (oaf_result) {
			CORBA_free (oaf_result);
		}
		
		CORBA_exception_free (&e);
	}

	if (!saved_ok) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_Project_IOFailure,
				     NULL);
	}
}

static void
file_project_destroy (GtkObject *object)
{
	Project *project;
	
	project = PROJECT (object);

	g_free (project->priv);
	project->priv = NULL;

	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
file_project_class_init (FileProjectClass *klass)
{
	GtkObjectClass *object_class = (GtkObjectClass *) klass;
	ProjectClass   *project_class = (ProjectClass *) klass;

	object_class->destroy = file_project_destroy;

	project_class->load                  = fp_load;
	project_class->save                  = fp_save;
}

Project *
file_project_new (void)
{
	return gtk_type_new (FILE_PROJECT_TYPE);
}

static void
file_project_init (FileProject *file_project)
{
	Project *project;

	project = PROJECT (file_project);
	
	/* Add a TaskManager object. */
	project->task_manager = file_task_manager_new (project->event_source);
 	bonobo_object_add_interface (BONOBO_OBJECT (project), 
				     BONOBO_OBJECT (project->task_manager));

	/* Add a ResourceManager object. */
	project->resource_manager = 
		file_resource_manager_new (project->event_source);
	bonobo_object_add_interface (BONOBO_OBJECT (project), 
				     BONOBO_OBJECT (project->resource_manager));

	/* Add an AllocationManager object. */
	project->allocation_manager = 
		file_allocation_manager_new (project->event_source);
	bonobo_object_add_interface (BONOBO_OBJECT (project),
				     BONOBO_OBJECT (project->allocation_manager));
	allocation_manager_start_listen (project->allocation_manager);
}
