/*
 * Copyright (C) 2001  CodeFactory AB
 * Copyright (C) 2001  Richard Hult
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult <rhult@codefactory.se>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <locale.h>
#include <math.h>
#include <stdlib.h>
#include <gal/widgets/e-unicode.h>
#include <bonobo.h>
#include <bonobo/bonobo-shlib-factory.h>
#include <liboaf/liboaf.h>
#include <libgnomevfs/gnome-vfs.h>
#include "gnome-xml/parser.h"
#include "gnome-xml/parserInternals.h"
#include "gnome-xml/xmlmemory.h"
#include "libmrproject/GNOME_MrProject.h"
#include "util/time-utils.h"
#include "util/type-utils.h"
#include "project-server/project.h"

#include "file-filter.h"
#include "xml-file-filter.h"

static void xml_file_filter_init	(XmlFileFilter	  *xml_file_filter);
static void xml_file_filter_class_init	(XmlFileFilterClass *klass);

GNOME_CLASS_BOILERPLATE (XmlFileFilter, xml_file_filter,
			 FileFilter, file_filter);

typedef struct _xmlParseContext xmlParseContext;
struct _xmlParseContext {
        xmlDocPtr            doc;      /* Xml document. */
	xmlNsPtr             ns;       /* Main namespace. */

	xmlNodePtr           tasks_node;
	xmlNodePtr           resource_groups_node;
	xmlNodePtr           resources_node;
 	xmlNodePtr           allocations_node;
	
	GM_Project           project;
	GM_TaskManager       task_mgr;
	GM_ResourceManager   resource_mgr;
	GM_AllocationManager allocation_mgr;

	TaskManager 	    *task_manager;
	ResourceManager     *resource_manager;
 	AllocationManager   *allocation_manager;
	
	GHashTable	    *id_hash;
	GSList              *delayed_dependencies;

	/* Keep track of the earliest/latest task. */
	time_t               t1, t2;
};

typedef struct {
	gint                           task_id;
	gint                           predecessor_id;
	gint                           dependency_id;
	GNOME_MrProject_DependencyType type;
} DelayedDependency;

static gchar      *xml_get_value       (xmlNodePtr node, const char *name);
static gchar      *xml_get_string_utf8 (xmlNodePtr node, const char *name);
static xmlNodePtr  xml_search_child    (xmlNodePtr node, const gchar *name);
static void        read_project_properties (xmlNodePtr node, GNOME_MrProject_Project project_co);
static void        read_start_time     (xmlParseContext *ctxt, GNOME_MrProject_Project project_co);
static void        write_project_properties (xmlNodePtr node, GNOME_MrProject_Project project_co);

static void
write_project_settings (xmlNodePtr cur, xmlParseContext *ctxt)
{

}

static void
write_predecessor (xmlParseContext            *ctxt,
		   xmlNodePtr                  parent,
		   GNOME_MrProject_Dependency *dependency)
{
	xmlNodePtr            node;
	gchar                *str;

	node = xmlNewChild (parent, ctxt->ns, "predecessor", NULL);
	
	if (!node){
		g_warning ("Failed saving predecessor.\n");
		return;
	}

	str = g_strdup_printf ("%d", dependency->depId);
	xmlSetProp (node, "id", str);
	g_free (str);

	str = g_strdup_printf ("%d", dependency->predecessorId);
	xmlSetProp (node, "predecessor-id", str);
	g_free (str);

	switch (dependency->type) {
	case GNOME_MrProject_DEPENDENCY_FS:
		str = "FS";
		break;
	case GNOME_MrProject_DEPENDENCY_FF:
		str = "FF";
		break;
	case GNOME_MrProject_DEPENDENCY_SF:
		str = "SF";
		break;
	case GNOME_MrProject_DEPENDENCY_SS:
		str = "SS";
		break;
	default:
		str = "FS";
	}
	xmlSetProp (node, "type", str);
}

static void
write_task (xmlParseContext *ctxt, GM_Task *task)
{
	xmlNodePtr                     node, parent_node, predecessors_node;
	gint                           parent_id, *key;
	gchar                         *str;
	CORBA_Environment              ev;
	GNOME_MrProject_DependencySeq *predecessors;
	guint                          i;

	g_return_if_fail (task != NULL);

	parent_id = task->parentId;
	parent_node = g_hash_table_lookup (ctxt->id_hash, &parent_id);

	/* This is just a precaution, should never be needed. */
	if (parent_node == NULL) {
		parent_node = ctxt->tasks_node;
	}
		
	node = xmlNewChild (parent_node, ctxt->ns, "task", NULL);
	if (!node){
		g_warning ("Failed saving task.");
		return;
	}

	key = g_new (gint, 1);
	*key = task->taskId;
	g_hash_table_insert (ctxt->id_hash, key, node);

	str = g_strdup_printf ("%d", task->taskId);
	xmlSetProp (node, "id", str);
	g_free (str);

	xmlSetProp (node, "name", task->name);

	str = isodate_from_time_t (task->start);
	xmlSetProp (node, "start", str);
	g_free (str);

	str = isodate_from_time_t (task->end);
	xmlSetProp (node, "end", str);
	g_free (str);

	CORBA_exception_init (&ev);
	predecessors = GNOME_MrProject_TaskManager_getPredecessors (
		ctxt->task_mgr, task->taskId, &ev);
	
	if (!BONOBO_EX (&ev) && predecessors) {
		predecessors_node = xmlNewChild (node, ctxt->ns, 
						 "predecessors", 
						 NULL);
		for (i = 0; i < predecessors->_length; i++) {
			write_predecessor (ctxt, predecessors_node,
					   &predecessors->_buffer[i]);
		}
	}

	CORBA_exception_free (&ev);
}

static void
write_resource (xmlParseContext *ctxt,
		GM_Resource     *resource)
{
	xmlNodePtr  node;
	gchar      *str;
	
	g_return_if_fail (resource != NULL);

	node = xmlNewChild (ctxt->resources_node, ctxt->ns, "resource", NULL);
	if (!node){
		g_warning ("Failed saving resource.");
		return;
	}

	str = g_strdup_printf ("%d", resource->resourceId);
	xmlSetProp (node, "id", str);
	g_free (str);

	xmlSetProp (node, "name", resource->name);
	str = g_strdup_printf ("%d", resource->groupId);
	xmlSetProp (node, "group", str);
	g_free (str);
	
	str = g_strdup_printf ("%d", resource->type);
	xmlSetProp (node, "type", str);
	g_free (str);

	str = g_strdup_printf ("%d", resource->units);
	xmlSetProp (node, "units", str);
	g_free (str);

	str = g_strdup_printf ("%f", resource->stdRate);
	xmlSetProp (node, "std-rate", str);
	g_free (str);

	str = g_strdup_printf ("%f", resource->ovtRate);
	xmlSetProp (node, "ovt-rate", str);
	g_free (str);
}

static void
write_resource_group (xmlParseContext    *ctxt, 
                      GM_ResourceGroup   *group)
{
	xmlNodePtr  node;
	gchar      *str;

	g_return_if_fail (ctxt != NULL);
	g_return_if_fail (group != NULL);

	node = xmlNewChild (ctxt->resource_groups_node, ctxt->ns, "group", NULL);
	if (!node){
		g_warning ("Failed saving resource group.");
		return;
	}

	str = g_strdup_printf ("%d", group->groupId);
	xmlSetProp (node, "id", str);
	g_free (str);
	
	xmlSetProp (node, "name", group->name);
	xmlSetProp (node, "admin-name", group->adminName);
	xmlSetProp (node, "admin-phone", group->adminPhone);
	xmlSetProp (node, "admin-email", group->adminEmail);
}

static void
write_allocation (xmlParseContext *ctxt, GM_Allocation *allocation)
{
	xmlNodePtr  node;
	gchar      *str;

	g_return_if_fail (ctxt != NULL);
	g_return_if_fail (allocation != NULL);

	node = xmlNewChild (ctxt->allocations_node, ctxt->ns,
			    "allocation", NULL);
	if (!node){
		g_warning ("Failed saving allocation.");
		return;
	}

	str = g_strdup_printf ("%d", allocation->taskId);
	xmlSetProp (node, "task-id", str);
	g_free (str);
	str = g_strdup_printf ("%d", allocation->resourceId);
	xmlSetProp (node, "resource-id", str);
	g_free (str);
}

static xmlNodePtr
xml_write_project (xmlParseContext *ctxt)
{
	xmlNodePtr           cur;
	xmlNsPtr             ns;
	gint                *key;
	gchar               *old_locale;
	CORBA_Environment    ev;
	GM_TaskSeq          *tasks;
	GM_ResourceSeq      *resources;
	GM_ResourceGroupSeq *resource_groups;
	GM_AllocationSeq    *allocations;
	guint                i;
        GM_Id                default_group;
        gchar               *str;
        
	g_return_val_if_fail (ctxt != NULL, NULL);

	cur = xmlNewDocNode (ctxt->doc, ctxt->ns, "project", NULL);
	if (cur == NULL) {
		return NULL;
	}
	
	if (ctxt->ns == NULL) {
		ns = xmlNewNs (cur, "http://mrproject.codefactory.se/format/project/1", NULL);
		xmlSetNs (cur, ns);
		ctxt->ns = ns;
	}

	/* Project properties. */
	write_project_properties (cur, ctxt->project);

	/* Make sure we don't get locale specific floats. */
	old_locale = setlocale (LC_NUMERIC, "C");

	write_project_settings (cur, ctxt);
	
	/* Write tasks */
	ctxt->tasks_node = xmlNewChild (cur, ctxt->ns, "tasks", NULL);
	key = g_new (gint, 1);
	*key = 0;
	g_hash_table_insert (ctxt->id_hash, key, ctxt->tasks_node);

	CORBA_exception_init (&ev);
	tasks = GNOME_MrProject_TaskManager_getAllTasks (ctxt->task_mgr, &ev);
	if (!BONOBO_EX (&ev) && tasks) {
		for (i = 0; i < tasks->_length; i++) {
			write_task (ctxt, &tasks->_buffer[i]);
		}
		CORBA_free (tasks);
	} else {
		CORBA_exception_free (&ev);
	}

	GNOME_MrProject_TaskManager__set_state (
		ctxt->task_mgr, GNOME_MrProject_PROJECT_STATE_SAVED, &ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
	}

	/* Write resource groups. */
	ctxt->resource_groups_node = xmlNewChild (cur, ctxt->ns, 
						  "resource-groups", 
						  NULL);

	resource_groups = GNOME_MrProject_ResourceManager_getAllGroups (
		ctxt->resource_mgr,
		&ev);
        
        default_group = GNOME_MrProject_ResourceManager_getDefaultGroup (
                ctxt->resource_mgr,
                &ev);

        if (default_group != -1) {
                str = g_strdup_printf ("%d", default_group);
                xmlSetProp (ctxt->resource_groups_node, 
                            "default_group", str);
                g_free (str);
        }
        

	if (!BONOBO_EX (&ev) && resource_groups) {
		for (i = 0; i < resource_groups->_length; i++) {
			write_resource_group (ctxt, 
                                              &resource_groups->_buffer[i]);
		}
		CORBA_free (resource_groups);
	} else {
		CORBA_exception_free (&ev);
	}

	/* Write resources. */
	ctxt->resources_node = xmlNewChild (cur, ctxt->ns, "resources", NULL);
	resources = GNOME_MrProject_ResourceManager_getAllResources (
		ctxt->resource_mgr, &ev);
	if (!BONOBO_EX (&ev) && resources) {
		for (i = 0; i < resources->_length; i++) {
			write_resource (ctxt, &resources->_buffer[i]);
		}
		CORBA_free (resources);
	} else {
		CORBA_exception_free (&ev);
	}

	GNOME_MrProject_ResourceManager__set_state (
		ctxt->resource_mgr, GNOME_MrProject_PROJECT_STATE_SAVED, &ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
	}

	/* Write allocations. */
	ctxt->allocations_node = xmlNewChild (cur, ctxt->ns,
					      "allocations", NULL);
	allocations = GNOME_MrProject_AllocationManager_getAllAllocations (
		ctxt->allocation_mgr, &ev);
	if (!BONOBO_EX (&ev) && allocations) {
		for (i = 0; i < allocations->_length; i++) {
			write_allocation (ctxt, &allocations->_buffer[i]);
		}
		CORBA_free (allocations);
	} else {
		CORBA_exception_free (&ev);
	}

	GNOME_MrProject_AllocationManager__set_state (
		ctxt->allocation_mgr, GNOME_MrProject_PROJECT_STATE_SAVED, &ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
	}

	/* Restore locale. */
	setlocale (LC_NUMERIC, old_locale);
 	return cur;
}

static xmlParseContext *
xml_parse_ctx_new (xmlDocPtr  doc,
		   GM_Project project)
{
	xmlParseContext   *ctxt;
	CORBA_Environment  ev;

	g_return_val_if_fail (doc != NULL, NULL);

	ctxt = g_new0 (xmlParseContext, 1);

	ctxt->doc = doc;
	ctxt->project = project;

	ctxt->t1 = G_MAXLONG;
	ctxt->t2 = -G_MAXLONG;
	
	CORBA_exception_init (&ev);
	ctxt->task_mgr = Bonobo_Unknown_queryInterface (
		project,
		"IDL:GNOME/MrProject/TaskManager:1.0",
		&ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not get TaskManager interface on project.");
		CORBA_exception_free (&ev);
	}

	ctxt->resource_mgr = Bonobo_Unknown_queryInterface (
		project,
		"IDL:GNOME/MrProject/ResourceManager:1.0",
		&ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not get ResourceManager interface on project.");
		CORBA_exception_free (&ev);
	}

	ctxt->allocation_mgr = Bonobo_Unknown_queryInterface (
		project,
		"IDL:GNOME/MrProject/AllocationManager:1.0",
		&ev);
	if (BONOBO_EX (&ev)) {
		g_warning ("Could not get AllocationManager interface on project.");
		CORBA_exception_free (&ev);
	}

	ctxt->id_hash = g_hash_table_new (g_int_hash, g_int_equal);
	ctxt->delayed_dependencies = NULL;

	return ctxt;
}

static void
id_hash_destroy (gpointer key, gpointer value, gpointer user_data)
{
	g_free (key);
}

static void
xml_parse_ctx_destroy (xmlParseContext *ctxt)
{
	if (ctxt->task_mgr)
		bonobo_object_release_unref (ctxt->task_mgr, NULL);
	if (ctxt->resource_mgr)
		bonobo_object_release_unref (ctxt->resource_mgr, NULL);
	if (ctxt->allocation_mgr)
		bonobo_object_release_unref (ctxt->allocation_mgr, NULL);
	
	g_hash_table_foreach (ctxt->id_hash, id_hash_destroy, NULL);
	g_hash_table_destroy (ctxt->id_hash);

	g_free (ctxt);
}

/*
 * Read
 */

static void
xml_read_predecessor (xmlParseContext    *ctxt,
		      GNOME_MrProject_Id  task_id,
		      xmlNodePtr          tree)
{
	gint               id, predecessor_id;
	DelayedDependency *dep;
	gchar             *str;

	if (strcmp (tree->name, "predecessor")){
		g_warning ("Got %s, expected 'predecessor'.", tree->name);
		return;
	}

	str = xml_get_value (tree, "id");
	if (str) {
		id = atoi (str);
		g_free (str);
	} else {
		id = -1;
	}

	str = xml_get_value (tree, "predecessor-id");
	if (str) {
		predecessor_id = atoi (str);
		g_free (str);
	} else {
		predecessor_id = -1;
	}

	if (id == -1 || predecessor_id == -1) {
		g_warning ("Invalid predecessor read.");
		return;
	}

	dep = g_new0 (DelayedDependency, 1);
	dep->task_id = task_id;
	dep->dependency_id = id;
	dep->predecessor_id = predecessor_id;

	str = xml_get_value (tree, "type");
	if (!strcmp (str, "FS")) {
		dep->type = GNOME_MrProject_DEPENDENCY_FS;
	}
	else if (!strcmp (str, "FF")) {
		dep->type = GNOME_MrProject_DEPENDENCY_FF;
	}
	else if (!strcmp (str, "SS")) {
		dep->type = GNOME_MrProject_DEPENDENCY_SS;
	}
	else if (!strcmp (str, "SF")) {
		dep->type = GNOME_MrProject_DEPENDENCY_SF;
	}
	else {
		dep->type = GNOME_MrProject_DEPENDENCY_FS;
	}

	ctxt->delayed_dependencies = g_slist_prepend (
		ctxt->delayed_dependencies,
		dep);
}

static void
xml_read_task (xmlParseContext *ctxt, xmlNodePtr tree, gint parent_id)
{
	xmlNodePtr                   tasks, predecessor;
	gchar                       *str, *name;
	gint                         id;
	time_t                       start, end;
	GNOME_MrProject_Task        *task;
	CORBA_Environment            ev;

	if (strcmp (tree->name, "task")){
		g_warning ("Got %s, expected 'task'.", tree->name);
		return;
	}

	name = xml_get_string_utf8 (tree, "name");

	str = xml_get_value (tree, "start");
	if (str) {
		start = time_from_isodate (str);
		g_free (str);
	} else {
		start = -1;
	}
	
	str = xml_get_value (tree, "end");
	if (str) {
		end = time_from_isodate (str);
		g_free (str);
	} else {
		end = -1;
	}
	
	str = xml_get_value (tree, "id");
	id = strtod (str, NULL);
	g_free (str);
	
	/* Create and insert a new row. */
	task = GNOME_MrProject_Task__alloc ();
	task->name = CORBA_string_dup (name);
	task->taskId = id;
	task->start = (GNOME_MrProject_Time) start;
	task->end = (GNOME_MrProject_Time) end;

	ctxt->t1 = MIN (ctxt->t1, start);
	ctxt->t2 = MAX (ctxt->t2, end);
	
	CORBA_exception_init (&ev);
	GNOME_MrProject_TaskManager_insertTask (
		ctxt->task_mgr, task, parent_id, &ev);
	CORBA_exception_free (&ev);

	CORBA_free (task);
	g_free (name);
	
	for (tasks = tree->childs; tasks; tasks = tasks->next) {
		if (!strcmp (tasks->name, "task")) {
			xml_read_task (ctxt, tasks, id);
		}
		else if (!strcmp (tasks->name, "predecessors")) {
			for (predecessor = tasks->childs; predecessor; predecessor = predecessor->next) {
				xml_read_predecessor (ctxt, id, predecessor);
			}
		}
	}
}

static void
xml_read_resource (xmlParseContext *ctxt, xmlNodePtr tree)
{
	GNOME_MrProject_Id                 id = 0;
	GNOME_MrProject_ResourceType       type = 0;
	gchar                             *str, *name;
	gint                               gid = 0;
	glong                              units = 0;
	gfloat                             std_rate = 0.0, ovt_rate = 0.0;
	GNOME_MrProject_Resource          *resource;
	CORBA_Environment                  ev;

	if (strcmp (tree->name, "resource")){
		g_warning ("Got %s, expected 'resource'.", tree->name);
		return;
	}

	str = xml_get_value (tree, "id");
	if (str) {
		id = atoi (str);
		g_free (str);
	} 

	name = xml_get_string_utf8 (tree, "name");

	str = xml_get_value (tree, "group");
	if (str) {
		gid = atoi (str);
		g_free (str);
	}

	str = xml_get_value (tree, "type");
	if (str) {
		type = atoi (str);
		g_free (str);
	}

	str = xml_get_value (tree, "units");
	if (str) {
		units = atol (str);
		g_free (str);
	}

	str = xml_get_value (tree, "std-rate");
	if (str) {
		std_rate = atof (str);
		g_free (str);
	}

	str = xml_get_value (tree, "ovt-rate");
	if (str) {
		ovt_rate = atof (str);
		g_free (str);
	}

	/* Create and insert a new row. */
	resource             = GNOME_MrProject_Resource__alloc ();
	resource->resourceId = id;
	resource->name       = CORBA_string_dup (name);
	resource->groupId    = gid;
	resource->type       = type;
	resource->units      = units;
	resource->stdRate    = std_rate;
	resource->ovtRate    = ovt_rate;

	CORBA_exception_init (&ev);
	
	GNOME_MrProject_ResourceManager_insertResource (ctxt->resource_mgr,
							resource,
							&ev);
	CORBA_exception_free (&ev);
	
	CORBA_free (resource);
	g_free (name);
}

static void
xml_read_group (xmlParseContext *ctxt, xmlNodePtr tree)
{
	gint                            id = 0;
	gchar                          *str;
	gchar                          *name, *a_name, *a_phone, *a_email;
	GNOME_MrProject_ResourceGroup  *group;
	CORBA_Environment               ev;
        
	if (strcmp (tree->name, "group")){
		g_warning ("Got %s, expected 'group'.", tree->name);
		return;
	}

	str = xml_get_value (tree, "id");
	if (str) {
		id = atoi (str);
		g_free (str);
	} 

	name    = xml_get_string_utf8 (tree, "name");
	a_name  = xml_get_string_utf8 (tree, "admin-name");
	a_phone = xml_get_string_utf8 (tree, "admin-phone");
	a_email = xml_get_string_utf8 (tree, "admin-email");

        /* Create and insert a new row. */
	group             = GNOME_MrProject_ResourceGroup__alloc ();
	group->groupId    = id;
	group->name       = CORBA_string_dup (name);
	group->adminName  = CORBA_string_dup (a_name);
	group->adminPhone = CORBA_string_dup (a_phone);
	group->adminEmail = CORBA_string_dup (a_email);

	CORBA_exception_init (&ev);
	
	GNOME_MrProject_ResourceManager_insertGroup (ctxt->resource_mgr,
						     group,
						     &ev);

	CORBA_exception_free (&ev);
	
	CORBA_free (group);
	g_free (name);
	g_free (a_name);
	g_free (a_phone);
	g_free (a_email);
}

static void
xml_read_allocation (xmlParseContext *ctxt, xmlNodePtr tree)
{
	gchar              *str;
	GNOME_MrProject_Id  task_id = -1, resource_id = -1;
	CORBA_Environment   ev;

	if (strcmp (tree->name, "allocation")){
		g_warning ("Got %s, expected 'allocation'.", tree->name);
		return;
	}

	str = xml_get_value (tree, "task-id");
	if (str) {
		task_id = atoi (str);
		g_free (str);
	}

	str = xml_get_value (tree, "resource-id");
	if (str) {
		resource_id = atoi (str);
		g_free (str);
	}
	
	CORBA_exception_init (&ev);
	
	GNOME_MrProject_AllocationManager_allocate (ctxt->allocation_mgr,
						    task_id,
						    resource_id,
						    &ev);
	CORBA_exception_free (&ev);
}

static void
read_project_properties (xmlNodePtr node, GNOME_MrProject_Project project_co)
{
	Bonobo_PropertyBag  pb_co;
	CORBA_Environment   ev;
	gchar              *str;

	CORBA_exception_init (&ev);	
	pb_co = Bonobo_Unknown_queryInterface (project_co,
					       "IDL:Bonobo/PropertyBag:1.0",
					       &ev);
	if (BONOBO_EX (&ev) || pb_co == CORBA_OBJECT_NIL) {
		g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);
		return;
 	}	

	str = xml_get_string_utf8 (node, "name");
	if (str != NULL) {
		bonobo_property_bag_client_set_value_string (
			pb_co, "Name", str, NULL);
		g_free (str);
	}

	str = xml_get_string_utf8 (node, "company");
	if (str != NULL) {
		bonobo_property_bag_client_set_value_string (
			pb_co, "Company", str, NULL);
		g_free (str);
	}

	str = xml_get_string_utf8 (node, "manager");
	if (str != NULL) {
		bonobo_property_bag_client_set_value_string (
			pb_co, "Manager", str, NULL);
		g_free (str);
	}
	
	bonobo_object_release_unref (pb_co, NULL);
	CORBA_exception_free (&ev);
}

/* This simply gets the start of the first task and sets
 * the Start property of the Project to that time.
 */
static void
read_start_time (xmlParseContext *ctxt, GNOME_MrProject_Project project_co)
{
	Bonobo_PropertyBag  pb_co;
	CORBA_Environment   ev;

	if (ctxt->t1 != G_MAXLONG) {
		CORBA_exception_init (&ev);	
		pb_co = Bonobo_Unknown_queryInterface (project_co,
						       "IDL:Bonobo/PropertyBag:1.0",
						       &ev);
		if (BONOBO_EX (&ev) || pb_co == CORBA_OBJECT_NIL) {
			g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
			CORBA_exception_free (&ev);
			return;
		}	
		
		bonobo_property_bag_client_set_value_glong (
			pb_co, "Start", ctxt->t1, NULL);
	
		bonobo_object_release_unref (pb_co, NULL);
		CORBA_exception_free (&ev);
	}
}

static void
write_project_properties (xmlNodePtr node, GM_Project project_co)
{
	gchar              *str;
	Bonobo_PropertyBag  pb_co;
	CORBA_Environment   ev;

	str = NULL;
	
	CORBA_exception_init (&ev);	
	pb_co = Bonobo_Unknown_queryInterface (project_co,
					       "IDL:Bonobo/PropertyBag:1.0",
					       &ev);
	if (BONOBO_EX (&ev) || pb_co == CORBA_OBJECT_NIL) {
		g_warning ("Error '%s'", bonobo_exception_get_text (&ev));
		CORBA_exception_free (&ev);
		return;
 	}	
	
	str = bonobo_property_bag_client_get_value_string (pb_co, "Name", NULL);

	xmlSetProp (node, "name", str);
	g_free (str);

	str = bonobo_property_bag_client_get_value_string (pb_co, "Company", NULL);
	xmlSetProp (node, "company", str);
	g_free (str);

	str = bonobo_property_bag_client_get_value_string (pb_co, "Manager", NULL);
	xmlSetProp (node, "manager", str);
	g_free (str);

	bonobo_object_release_unref (pb_co, NULL);
	CORBA_exception_free (&ev);
}

static void
process_delayed_predecessors (xmlParseContext *ctxt)
{
	GSList            *list;
	CORBA_Environment  ev;

	CORBA_exception_init (&ev);
	for (list = ctxt->delayed_dependencies; list; list = list->next) {
		DelayedDependency *dep;

		dep = list->data;

		GNOME_MrProject_TaskManager_linkTasks (ctxt->task_mgr,
						       dep->task_id,
						       dep->predecessor_id,
						       dep->type,
						       &ev);
		if (BONOBO_EX (&ev)) {
			g_warning ("Could not add dependency.");
		}
		CORBA_exception_free (&ev);
		g_free (dep);
	}

	g_slist_free (ctxt->delayed_dependencies);
	ctxt->delayed_dependencies = NULL;
}

static gboolean
xml_read_project (xmlParseContext *ctxt)
{
	xmlNodePtr         top, child, tasks, resources, groups, allocations;
	gchar             *old_locale;
	CORBA_Environment  ev;
        gchar             *str;
        GM_Id              default_group = -1;
	
	g_return_val_if_fail (ctxt != NULL, FALSE);
	g_return_val_if_fail (ctxt->doc != NULL, FALSE);
	g_return_val_if_fail (ctxt->doc->root != NULL, FALSE);

	top = ctxt->doc->root;
	if (!top) {
		g_warning ("Could not find project");
		return FALSE;
	}
	
	CORBA_exception_init (&ev);
	
	read_project_properties (top, ctxt->project);
	
	/* Make sure we don't get locale specific floats. */
	old_locale = setlocale (LC_NUMERIC, "C");

	/* Load tasks. */
	child = xml_search_child (top, "tasks");
	if (child != NULL) {
		for (tasks = child->childs; tasks; tasks = tasks->next) {
			xml_read_task (ctxt, tasks, 0 /* parent_id */);
		}
	}
	
	read_start_time (ctxt, ctxt->project);

	/* Now that all tasks are loaded, we can add the dependencies. */
	process_delayed_predecessors (ctxt);

	GNOME_MrProject_TaskManager__set_state (
		ctxt->task_mgr, GNOME_MrProject_PROJECT_STATE_SAVED, &ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
                return FALSE;
	}

	/* Load resources. */
 	child = xml_search_child (top, "resources"); 
 	if (child != NULL) {
		for (resources = child->childs; resources; resources = resources->next) {
			xml_read_resource (ctxt, resources); 
		}
	}
	
	/* Load resource groups. */
 	child = xml_search_child (top, "resource-groups"); 
        
        str = xml_get_value (child, "default_group");
        
        if (str) {
                default_group = atoi (str);
                g_free (str);
        }
        
        
        
 	if (child != NULL) {
		for (groups = child->childs; groups; groups = groups->next) {
			xml_read_group (ctxt, groups); 
		}
	}

        if (default_group > 0) {
                GNOME_MrProject_ResourceManager_setDefaultGroup (ctxt->resource_mgr,
                                                                 default_group,
                                                                 &ev);
        }

	GNOME_MrProject_ResourceManager__set_state (
		ctxt->resource_mgr, GNOME_MrProject_PROJECT_STATE_SAVED, &ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
                return FALSE;
	}

	/* Load allocations. */
	child = xml_search_child (top, "allocations");
	if (child != NULL) {
		for (allocations = child->childs; allocations; allocations = allocations->next) {
			xml_read_allocation (ctxt, allocations);
		}
	}

	GNOME_MrProject_AllocationManager__set_state (
		ctxt->allocation_mgr, GNOME_MrProject_PROJECT_STATE_SAVED, &ev);
	if (BONOBO_EX (&ev)) {
		CORBA_exception_free (&ev);
                return FALSE;
	}

	/* Restore locale. */
	setlocale (LC_NUMERIC, old_locale);

	return TRUE;
}

/*
 * Get a value for a node either carried as an attibute or as
 * the content of a child.
 *
 * Returns a g_malloc'ed string.  Caller must free.
 */
static gchar *
xml_get_value (xmlNodePtr node, const char *name)
{
	char *ret, *val;
	xmlNodePtr child;

	g_return_val_if_fail (node != NULL, NULL);
	
	val = (char *) xmlGetProp (node, name);
	if (val != NULL) {
		ret = g_strdup (val);
		xmlFree (val);
		return ret;
	}

	child = node->childs;

	while (child != NULL) {
		if (!strcmp (child->name, name)) {
		        /*
			 * !!! Inefficient, but ...
			 */
			val = xmlNodeGetContent(child);
			if (val != NULL) {
				ret = g_strdup (val);
				xmlFree (val);
				return ret;
			}
		}
		child = child->next;
	}

	return NULL;
}

static gchar *
xml_get_string_utf8 (xmlNodePtr node, const char *name)
{
	gchar *value, *utf8;

	value = xml_get_value (node, name);
	utf8 = e_utf8_from_locale_string (value);
	g_free (value);
	return utf8;
}

/*
 * Search a child by name, if needed go down the tree to find it.
 */
static xmlNodePtr
xml_search_child (xmlNodePtr node, const gchar *name)
{
	xmlNodePtr ret;
	xmlNodePtr child;

	child = node->childs;
	while (child != NULL) {
		if (!strcmp (child->name, name))
			return child;
		child = child->next;
	}
	child = node->childs;
	while (child != NULL) {
		ret = xml_search_child (child, name);
		if (ret != NULL)
			return ret;
		child = child->next;
	}
	return NULL;
}



static gboolean
xff_probe (FileFilter        *filter,
	   const gchar       *method,
	   const gchar       *location,
	   CORBA_Environment *ev)
{
	return TRUE;
}

static void
xff_load (FileFilter        *filter,
	  const gchar       *method,
	  const gchar       *location,
	  const GM_Project   project,
	  CORBA_Environment *ev)
{
	xmlDoc              *doc;
	xmlParseContext     *ctxt;
	xmlParserCtxtPtr     push_ctxt;

	GnomeVFSResult	     result;
	GnomeVFSHandle      *handle;
	GnomeVFSFileSize     bytes_read;
	gchar                buffer[1024];

	gchar *uri;

	/* FIXME: remove this and take the uri as argument. */
	uri = g_strconcat (method, ":", location, NULL);
		
	result = gnome_vfs_open	(&handle,
				 uri,
				 GNOME_VFS_OPEN_READ |
				 GNOME_VFS_OPEN_RANDOM);

	if (result != GNOME_VFS_OK) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}

	g_free (uri);
	
	/* Load the file into an XML tree. */
	push_ctxt = xmlCreatePushParserCtxt (NULL, NULL, NULL, 0, NULL);
	
	while (1) {
		result = gnome_vfs_read	(handle,
					 buffer,
					 1024,
					 &bytes_read);

		if (result == GNOME_VFS_ERROR_EOF) {
			break;
		}			
		else if (result != GNOME_VFS_OK) {
			break;
		}
			
		if (xmlParseChunk (push_ctxt, buffer, bytes_read, (bytes_read == 0))) {
			g_warning ("Leak bits of tree everywhere");
			break;
		}
	}
	
	gnome_vfs_close (handle);
	
	if (push_ctxt->wellFormed) {
		doc = push_ctxt->myDoc;
	} else {
		doc = NULL;
		xmlFreeDoc (push_ctxt->myDoc);
		push_ctxt->myDoc = NULL;
	}
	
	xmlFreeParserCtxt (push_ctxt);

	/*doc = xmlParseFile (location);*/
	if (!doc) {
		g_warning ("Failed to parse file.");
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}
	if (!doc->root) {
		xmlFreeDoc (doc);
		g_warning ("Invalid xml file. Tree is empty?");
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}

	/* FIXME: Check the namespace here? */
	ctxt = xml_parse_ctx_new (doc, project);

	if (!xml_read_project (ctxt)) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
	}
	
	xml_parse_ctx_destroy (ctxt);
	xmlFreeDoc (doc);
}

static void
xff_load_summary (FileFilter        *filter,
		  const gchar       *method,
		  const gchar       *location,
		  GNOME_MrProject_FileSummary **summary,
		  CORBA_Environment *ev)
{
	xmlDoc              *doc;
	xmlParserCtxtPtr     push_ctxt;
	GnomeVFSResult	     result;
	GnomeVFSHandle      *handle;
	GnomeVFSFileSize     bytes_read;
	gchar                buffer[1024];
	gchar               *uri;

	gchar               *str;
	time_t               t;

	/* FIXME: remove this and take the uri as argument. */
	uri = g_strconcat (method, ":", location, NULL);
		
	result = gnome_vfs_open	(&handle,
				 uri,
				 GNOME_VFS_OPEN_READ |
				 GNOME_VFS_OPEN_RANDOM);

	if (result != GNOME_VFS_OK) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}

	g_free (uri);

	/* Load the file into an XML tree. */
	push_ctxt = xmlCreatePushParserCtxt (NULL, NULL, NULL, 0, NULL);
	
	while (1) {
		result = gnome_vfs_read	(handle,
					 buffer,
					 1024,
					 &bytes_read);

		if (result == GNOME_VFS_ERROR_EOF) {
			break;
		}			
		else if (result != GNOME_VFS_OK) {
			break;
		}
			
		if (xmlParseChunk (push_ctxt, buffer, bytes_read, (bytes_read == 0))) {
			g_warning ("Leak bits of tree everywhere");
			break;
		}
	}

	gnome_vfs_close (handle);
	
	if (push_ctxt->wellFormed) {
		doc = push_ctxt->myDoc;
	} else {
		doc = NULL;
		xmlFreeDoc (push_ctxt->myDoc);
		push_ctxt->myDoc = NULL;
	}
	
	xmlFreeParserCtxt (push_ctxt);

	if (!doc) {
		g_warning ("Failed to parse file.");
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}
	if (!doc->root) {
		xmlFreeDoc (doc);
		g_warning ("Invalid xml file. Tree is empty?");
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}

	/* FIXME: Check the namespace here? */
	
	/* Ok, we have the file, now fetch the interesting pieces for the
	 * summary.
	 */

	*summary = GNOME_MrProject_FileSummary__alloc ();

	str = xml_get_string_utf8 (doc->root, "name");
	(**summary).name = CORBA_string_dup (str);
	g_free (str);

	str = xml_get_string_utf8 (doc->root, "company");
	(**summary).company = CORBA_string_dup (str);
	g_free (str);

	str = xml_get_string_utf8 (doc->root, "manager");
	(**summary).manager = CORBA_string_dup (str);
	g_free (str);

	str = xml_get_value (doc->root, "start");
	t = time_from_isodate (str);
	g_free (str);
	(**summary).start = t;

	str = xml_get_value (doc->root, "end");
	t = time_from_isodate (str);
	g_free (str);
	(**summary).end = t;

/*	{
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
	}
*/
	xmlFreeDoc (doc);
}

static void
xff_save (FileFilter        *filter,
	  const gchar       *method,
	  const gchar       *location,
	  const GM_Project   project,
	  CORBA_Environment *ev)
{
	int              ret;
	xmlDocPtr        xml;
	xmlParseContext *ctxt;

	xml = xmlNewDoc ("1.0");
	if (!xml) {
		g_warning ("Error creating xml tree.");
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
		return;
	}

	ctxt = xml_parse_ctx_new (xml, project);

	xml->root = xml_write_project (ctxt);
	xml_parse_ctx_destroy (ctxt);

	ret = xmlSaveFile (location, xml);
	if (ret == -1) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_FileFilter_Failure,
				     NULL);
	}
	
	xmlFreeDoc (xml);
}

static void
xml_file_filter_class_init (XmlFileFilterClass *klass)
{
	FileFilterClass *file_filter_class = (FileFilterClass *) klass;

	file_filter_class->probe        = xff_probe;
	file_filter_class->load         = xff_load;
	file_filter_class->load_summary = xff_load_summary;
	file_filter_class->save         = xff_save;
}

static void
xml_file_filter_init (XmlFileFilter *xml_file_filter)
{
}

static BonoboObject *
factory (BonoboGenericFactory *this,
	 const char           *object_id,
	 void                 *data)
{
	g_return_val_if_fail (object_id != NULL, NULL);
	
	if (!strcmp (object_id, "OAFIID:GNOME_MrProject_XmlFileFilter")) {
		FileFilter *filter;

		filter = gtk_type_new (TYPE_XML_FILE_FILTER);
		return BONOBO_OBJECT (filter);
	} else {
		g_warning ("Failing to manufacture a '%s'", object_id);
	}
	
	return NULL;
}

BONOBO_OAF_SHLIB_FACTORY_MULTI ("OAFIID:GNOME_MrProject_XmlFileFilterFactory",
				"Mr Project XML file filter factory",
				factory,
				NULL);

