/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2000 CodeFactory AB
 * Copyright (C) 2000 Richard Hult <rhult@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <bonobo.h>
#include <liboaf/liboaf.h>
#include "util/type-utils.h"
#include "project-factory.h"
#include "project.h"
#include "backends/file-project.h"

#define PROJECT_FACTORY_OAFIID "OAFIID:GNOME_MrProject_ProjectFactory"

static void project_factory_class_init (ProjectFactoryClass *klass);
static void project_factory_init       (ProjectFactory      *factory);
static void project_factory_destroy    (GtkObject           *object);


GNOME_CLASS_BOILERPLATE (ProjectFactory, project_factory,
			 BonoboGenericFactory, bonobo_generic_factory);


static POA_GNOME_MrProject_ProjectFactory__vepv project_factory_vepv;

struct _ProjectFactoryPriv {
	gint     ref_count;
	Project *debug_project;
	GList   *projects; /* List of all open project bonobo objects. */
};

static void
project_destroyed (GtkObject *object, ProjectFactory *pf)
{
	Project *project;

	puts (__FUNCTION__);

	project = PROJECT (object);

	pf->priv->projects = g_list_remove (pf->priv->projects, project);
}

static BonoboObject *
project_fn (BonoboGenericFactory *factory, void *closure)
{
	Project        *project;
	ProjectFactory *project_factory;

	project_factory = PROJECT_FACTORY (factory);

	/* Create a file backend for now... */
	project = file_project_new ();
	project_factory->priv->debug_project = project;

	project_factory->priv->projects = g_list_prepend (
		project_factory->priv->projects, project);

	gtk_signal_connect (GTK_OBJECT (project), 
			    "destroy",
			    project_destroyed,
			    project_factory);
	
	return BONOBO_OBJECT (project);	
}

POA_GNOME_MrProject_ProjectFactory__epv *
project_factory_get_epv (void)
{
	POA_GNOME_MrProject_ProjectFactory__epv *epv;
	
	epv = g_new0 (POA_GNOME_MrProject_ProjectFactory__epv, 1);

	return epv;
}

static void
ref (PortableServer_Servant  servant,
     CORBA_Environment      *ev)
{
	ProjectFactory *factory = PROJECT_FACTORY (bonobo_object_from_servant (servant));

	factory->priv->ref_count++;
}

static void
unref (PortableServer_Servant  servant,
       CORBA_Environment      *ev)
{
	ProjectFactory *factory = PROJECT_FACTORY (bonobo_object_from_servant (servant));

	factory->priv->ref_count--;
	if (factory->priv->ref_count <= 0) {
		/* Exit. */
		g_message ("Shutting down.");
		gtk_main_quit ();
	}
}

static GNOME_MrProject_ProjectSeq *
get_projects (PortableServer_Servant  servant,
	      CORBA_Environment      *ev)
{
	ProjectFactory             *pf;
	Project                    *project;
	GList                      *list;
	GNOME_MrProject_ProjectSeq *projects;
	GNOME_MrProject_Project     project_co;
	guint                       len, i;

	pf = PROJECT_FACTORY (bonobo_object_from_servant (servant));

	list = pf->priv->projects;
	len = g_list_length (list);
	
	projects           = GNOME_MrProject_ProjectSeq__alloc ();
	projects->_buffer  = CORBA_sequence_GNOME_MrProject_Project_allocbuf (len);
	projects->_length  = len;
	projects->_maximum = len;
	CORBA_sequence_set_release (projects, FALSE);

	for (i = 0; i < len; i++, list = list->next) {
		project = list->data;
		
		bonobo_object_ref (BONOBO_OBJECT (project));
		project_co = CORBA_Object_duplicate (BONOBO_OBJREF (project), NULL);

		projects->_buffer[i] = project_co;
	}

	return projects;
}

static GNOME_MrProject_Project
get_last_project (PortableServer_Servant  servant,
		  CORBA_Environment      *ev)
{
	ProjectFactory *factory = PROJECT_FACTORY (bonobo_object_from_servant (servant));
	Project        *project;
	BonoboObject   *object;

	project = factory->priv->debug_project;
	
	if (project && IS_PROJECT (project)) { 
		object = BONOBO_OBJECT (project);
		bonobo_object_ref (object);
		return CORBA_Object_duplicate (bonobo_object_corba_objref (object), ev);
	} else {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_ProjectFactory_NoProject,
				     NULL);
		return CORBA_OBJECT_NIL;
	}
}

static void
init_corba_class (void)
{
	project_factory_vepv.GNOME_ObjectFactory_epv = bonobo_generic_factory_get_epv ();
	project_factory_vepv.GNOME_MrProject_ProjectFactory_epv = project_factory_get_epv ();

	project_factory_vepv.GNOME_ObjectFactory_epv->ref = ref;
	project_factory_vepv.GNOME_ObjectFactory_epv->unref = unref;

	project_factory_vepv.GNOME_MrProject_ProjectFactory_epv->getProjects = get_projects;
	project_factory_vepv.GNOME_MrProject_ProjectFactory_epv->getLastProject = get_last_project;
}

static void
project_factory_class_init (ProjectFactoryClass *klass)
{
	GtkObjectClass *object_class = (GtkObjectClass *) klass;

	object_class->destroy = project_factory_destroy;

	init_corba_class ();
}

static void
project_factory_init (ProjectFactory *factory)
{
	factory->priv = g_new0 (ProjectFactoryPriv, 1);
}

void
project_factory_construct (ProjectFactory                 *factory,
			   GNOME_MrProject_ProjectFactory  corba_factory)
{
	g_return_if_fail (factory != NULL);
	g_return_if_fail (IS_PROJECT_FACTORY (factory));
	g_return_if_fail (corba_factory != CORBA_OBJECT_NIL);

	/* Call parent constructor. */
	bonobo_generic_factory_construct (PROJECT_FACTORY_OAFIID,
					  BONOBO_GENERIC_FACTORY (factory),
					  corba_factory,
					  project_fn,
					  NULL,
					  NULL);
}

/*
 * This routine creates the CORBA server and initializes the
 * CORBA side of things.
 */
static GNOME_MrProject_ProjectFactory
create_project_factory (BonoboObject *factory)
{
	POA_GNOME_MrProject_ProjectFactory *servant;
	CORBA_Environment                   ev;

	servant = (POA_GNOME_MrProject_ProjectFactory *) g_new0 (BonoboObjectServant, 1);
	servant->vepv = &project_factory_vepv;

	CORBA_exception_init (&ev);
	POA_GNOME_MrProject_ProjectFactory__init ((PortableServer_Servant) servant, &ev);

	if (ev._major != CORBA_NO_EXCEPTION){
		g_message ("project factory: Could not init the servant.");
		g_free (servant);
		CORBA_exception_free (&ev);
		return CORBA_OBJECT_NIL;
	}

	CORBA_exception_free (&ev);

	/*
	 * Activate the CORBA object & bind to the servant.
	 */
	return (GNOME_MrProject_ProjectFactory) bonobo_object_activate_servant (factory, servant);
}

ProjectFactory *
project_factory_new (void)
{
	ProjectFactory                 *factory;
	GNOME_MrProject_ProjectFactory  corba_factory;
	CORBA_Environment               ev;
	gboolean                        is_nil;

	factory = gtk_type_new (project_factory_get_type ());

	corba_factory = create_project_factory (BONOBO_OBJECT (factory));

	CORBA_exception_init (&ev);
	is_nil = CORBA_Object_is_nil (corba_factory, &ev);

	if (ev._major != CORBA_NO_EXCEPTION || is_nil) {
		g_message ("project_factory_new(): could not create the CORBA factory");
		bonobo_object_unref (BONOBO_OBJECT (factory));
		CORBA_exception_free (&ev);
		return NULL;
	}

	CORBA_exception_free (&ev);

	project_factory_construct (factory, corba_factory);	

	return factory;
}

static void
project_factory_destroy (GtkObject *object)
{
	GTK_OBJECT_CLASS (parent_class)->destroy (object);
}


