/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 * Copyright (C) 2000-2001 CodeFactory AB
 * Copyright (C) 2000-2001 Richard Hult <rhult@codefactory.se>
 * Copyright (C) 2001      Mikael Hallendal <micke@codefactory.se>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Richard Hult
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <bonobo.h>
#include <liboaf/liboaf.h>
#include <orb/orbit.h>
#include <gtk/gtkobject.h>
#include <gnome-xml/parser.h>
#include <gnome-xml/parserInternals.h>
#include <gnome-xml/xmlmemory.h>
#include "util/string-utils.h"
#include "util/time-utils.h"
#include "task-manager.h"
#include "resource-manager.h"
#include "allocation-manager.h"
#include "backends/file-task-manager.h"
#include "backends/file-resource-manager.h"
#include "backends/file-allocation-manager.h"
#include "project.h"

#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
static GtkObjectClass *parent_class;

/* Bonobo properties. */
enum {
	PROP_START,
	PROP_FINISH,
	PROP_NAME,
	PROP_MANAGER,
	PROP_COMPANY
};

static void project_init       (Project      *project);
static void project_class_init (ProjectClass *klass);


/* Private members. */
struct _ProjectPriv {
	BonoboPropertyBag   *property_bag;
	/*BonoboPersistStream *stream;*/
	GList               *shells;
	gchar               *uri;

	/* Properties. */
	gchar 		    *name, *company, *manager;
	time_t		     start, finish;
};

typedef struct {
	GNOME_MrProject_Shell shell_co;
	gulong                listener_id;
} ShellInfo;

static void
impl_Project_addShell (PortableServer_Servant       servant, 
		       const GNOME_MrProject_Shell  shell, 
		       CORBA_Environment           *ev)
{
	Project            *project;
	ProjectPriv        *priv;
	Bonobo_Listener     listener_co;
	Bonobo_EventSource  event_source_co;
	glong               id;
	ShellInfo          *info;

	project = PROJECT (bonobo_x_object (servant));
	priv = project->priv;

	listener_co = Bonobo_Unknown_queryInterface (shell,
						     "IDL:Bonobo/Listener:1.0",
						     ev);
	if (listener_co == CORBA_OBJECT_NIL || BONOBO_EX (ev)) {
		return;
	}

	event_source_co = BONOBO_OBJREF (project->event_source);
	id = Bonobo_EventSource_addListener (event_source_co,
					     listener_co,
					     CORBA_OBJECT_NIL);
	bonobo_object_release_unref (listener_co, CORBA_OBJECT_NIL);

	/* Add the Shell's listener to the project property bag as well. */
	event_source_co = BONOBO_OBJREF (priv->property_bag->es);
	Bonobo_EventSource_addListener (event_source_co,
					listener_co,
					CORBA_OBJECT_NIL);

	info = g_new0 (ShellInfo, 1);
	info->shell_co = CORBA_Object_duplicate (shell, ev);
	info->listener_id = id;
	priv->shells = g_list_prepend (priv->shells, info);
}

static void
impl_Project_removeShell (PortableServer_Servant       servant, 
			  const GNOME_MrProject_Shell  shell, 
			  CORBA_Environment           *ev)
{
	Project            *project;
	ProjectPriv        *priv;
	gboolean            found;
	GList              *list;
	ShellInfo          *info;
	Bonobo_EventSource  event_source_co;

	project = PROJECT (bonobo_object_from_servant (servant));
	priv = project->priv;

	found = FALSE;
	info = NULL;
	for (list = priv->shells; list; list = list->next) {
		info = list->data;
		if (CORBA_Object_is_equivalent (info->shell_co, shell, ev)) {
			priv->shells = g_list_remove (priv->shells, info);
			found = TRUE;
			break;
		}
	}
	
	if (!found) {
		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
				     ex_GNOME_MrProject_Project_NoSuchShell,
				     NULL);
		return;
	}

	event_source_co = BONOBO_OBJREF (project->event_source);
	Bonobo_EventSource_removeListener (event_source_co,
					   info->listener_id,
					   CORBA_OBJECT_NIL);
	g_free (info);
}

static GNOME_MrProject_ProjectState 
impl_Project_getState (PortableServer_Servant  servant,
		       CORBA_Environment      *ev)
{
	Project                      *project;
	ProjectPriv                  *priv;
	GNOME_MrProject_ProjectState  a_state, r_state, t_state;

	project  = PROJECT (bonobo_x_object (servant));
	priv     = project->priv;

	a_state = r_state = t_state = GNOME_MrProject_PROJECT_STATE_EMPTY;

	gtk_object_get (GTK_OBJECT (project->allocation_manager),
			"state",
			&a_state,
			NULL);

	gtk_object_get (GTK_OBJECT (project->resource_manager),
			"state",
			&r_state,
			NULL);

	gtk_object_get (GTK_OBJECT (project->task_manager),
			"state",
			&t_state,
			NULL);
	
	if (a_state == GNOME_MrProject_PROJECT_STATE_EMPTY &&
	    r_state == GNOME_MrProject_PROJECT_STATE_EMPTY &&
	    t_state == GNOME_MrProject_PROJECT_STATE_EMPTY) {
		return GNOME_MrProject_PROJECT_STATE_EMPTY;
	}

	if (a_state == GNOME_MrProject_PROJECT_STATE_DIRTY ||
	    r_state == GNOME_MrProject_PROJECT_STATE_DIRTY ||
	    t_state == GNOME_MrProject_PROJECT_STATE_DIRTY) {
		return GNOME_MrProject_PROJECT_STATE_DIRTY;
	}

	return GNOME_MrProject_PROJECT_STATE_SAVED;
}

static void
impl_Project_load (PortableServer_Servant  servant,
		   const CORBA_char       *uri,
		   CORBA_Environment      *ev)
{
	Project	           *project;
	gchar              *method, *location;
	ProjectClass       *klass;	

	project = PROJECT (bonobo_x_object (servant));
	klass   = PROJECT_CLASS (GTK_OBJECT (project)->klass);

	method   = NULL;
	location = NULL;
	if (!split_uri (uri, &method, &location)) {
		g_free (method);
		g_free (location);
		return;
	}	

	klass->load (project, method, location, ev);
	if (BONOBO_EX (ev)) {
		/* Could not load. */
	} else {
		project->priv->uri = g_strdup (uri);
	}
	
	g_free (method);
	g_free (location);
}

static void
impl_Project_save (PortableServer_Servant  servant,
		   const CORBA_char       *uri,
		   CORBA_Environment      *ev)
{
	Project      *project;
	gchar        *method, *location;
	ProjectClass *klass;
	
	project  = PROJECT (bonobo_x_object (servant));
	method   = NULL;
	location = NULL;

	klass = PROJECT_CLASS (GTK_OBJECT (project)->klass);

	if ((uri == CORBA_OBJECT_NIL || strlen (uri) == 0)) {
		if (!project->priv->uri) { /* No new URI and no old one. */
			/* FIXME: raise an exception if we
			 * don't have a valid location.
			 */
			return;
		}
	} else if (!project->priv->uri) {
		/* New URI, but no old one -> use the new one. */
		project->priv->uri = g_strdup (uri);
	} else if (strcmp (uri, project->priv->uri)) {
		/* New URI and old one that differs ->
		 * free old one and use new one.
		 */
		g_free (project->priv->uri);
		project->priv->uri = g_strdup (uri);
	}
	
	/* FIXME: We don't use the method for now. */
	if (!split_uri (project->priv->uri, &method, &location)) {
		g_free (method);
		g_free (location);
		g_warning ("Malformed URI.");
		return;
	}

	klass->save (project, method, location, ev);
	
	g_free (method);
	g_free (location);
}

static CORBA_char *
impl_Project_getURI (PortableServer_Servant  servant,
		     CORBA_Environment      *ev)
{
	Project *project;
	
	project = PROJECT (bonobo_x_object (servant));
	return CORBA_string_dup (project->priv->uri);
}

static void
impl_Project_setURI (PortableServer_Servant  servant,
		     const CORBA_char       *uri,
		     CORBA_Environment      *ev)
{
	Project *project;
	
	project = PROJECT (bonobo_x_object (servant));

	if (project->priv->uri) {
		CORBA_free (project->priv->uri);
	}
		
	project->priv->uri = CORBA_string_dup (uri);
}

static void
project_destroy (GtkObject *object)
{
	Project *project;

	project = PROJECT (object);

	g_free (project->priv);
	project->priv = NULL;

	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

static void
project_class_init (ProjectClass *klass)
{
	POA_GNOME_MrProject_Project__epv *epv = &klass->epv;
	GtkObjectClass *object_class = (GtkObjectClass *) klass;

	parent_class = gtk_type_class (PARENT_TYPE);

	object_class->destroy = project_destroy;

	epv->addShell         = impl_Project_addShell;
	epv->removeShell      = impl_Project_removeShell;
	epv->getState         = impl_Project_getState;
	epv->load             = impl_Project_load;
	epv->save             = impl_Project_save;
	epv->getURI           = impl_Project_getURI;
	epv->setURI           = impl_Project_setURI;
}

BONOBO_X_TYPE_FUNC_FULL (Project, 
			 GNOME_MrProject_Project,
			 PARENT_TYPE,
			 project);

static void
set_prop (BonoboPropertyBag *bag,
	  const BonoboArg   *arg,
	  guint              arg_id,
	  CORBA_Environment *ev,
	  gpointer           user_data)
{
	Project *project;
	gchar   *str;
	time_t   t;

	g_return_if_fail (user_data != NULL);
	g_return_if_fail (IS_PROJECT (user_data));

	project = PROJECT (user_data);
	
	switch (arg_id) {
	case PROP_NAME:
		str = BONOBO_ARG_GET_STRING (arg);
		g_free (project->priv->name);
		project->priv->name = g_strdup (str);
		break;

	case PROP_COMPANY:
		str = BONOBO_ARG_GET_STRING (arg);
		g_free (project->priv->company);
		project->priv->company = g_strdup (str);
		break;

	case PROP_MANAGER:
		str = BONOBO_ARG_GET_STRING (arg);
		g_free (project->priv->manager);
		project->priv->manager = g_strdup (str);
		break;

	case PROP_START:
		t = BONOBO_ARG_GET_INT (arg);
		project->priv->start = t;
		break;

	case PROP_FINISH:
		t = BONOBO_ARG_GET_INT (arg);
		project->priv->finish = t;
		break;
		
	default:
		g_print ("Project FIXME: set prop %d unimplemented.\n", arg_id);
		break;
	}
}

static void
get_prop (BonoboPropertyBag *bag,
	  BonoboArg         *arg,
	  guint              arg_id,
	  CORBA_Environment *ev,
	  gpointer           user_data)
{
	Project *project;
	gchar   *str;

	g_return_if_fail (user_data != NULL);
	g_return_if_fail (IS_PROJECT (user_data));

	project = PROJECT (user_data);
	switch (arg_id) {
	case PROP_START:
		BONOBO_ARG_SET_GENERAL (arg,
					project->priv->start,
					TC_CORBA_long,
					GNOME_MrProject_Time,
					NULL);
		break;

	case PROP_FINISH:
		BONOBO_ARG_SET_GENERAL (arg,
					project->priv->finish,
					TC_CORBA_long,
					GNOME_MrProject_Time,
					NULL);
		break;

	case PROP_NAME:
		str = project->priv->name;
		BONOBO_ARG_SET_STRING (arg, str); 
		break;

	case PROP_COMPANY:
		str = project->priv->company;
		BONOBO_ARG_SET_STRING (arg, str); 
		break;

	case PROP_MANAGER:
		str = project->priv->manager;
		BONOBO_ARG_SET_STRING (arg, str); 
		break;

	default:
		break;
	}
}

static void
project_init (Project *project)
{
	ProjectPriv       *priv;
	BonoboPropertyBag *property_bag;
	BonoboEventSource *event_source;
	
	priv = g_new0 (ProjectPriv, 1);
	project->priv = priv;

	priv->uri     = g_strdup ("");
	priv->name    = g_strdup ("");
	priv->company = g_strdup ("");
	priv->manager = g_strdup ("");

	property_bag = bonobo_property_bag_new (get_prop, set_prop, project);
	bonobo_object_add_interface (BONOBO_OBJECT (project), BONOBO_OBJECT (property_bag));
	bonobo_property_bag_add (property_bag,
				 "Start",
				 PROP_START,
				 TC_CORBA_long,
				 NULL,
				 "The project start date",
				 0);
	bonobo_property_bag_add (property_bag,
				 "Finish",
				 PROP_FINISH,
				 TC_CORBA_long,
				 NULL,
				 "The project finish date",
				 0);
	bonobo_property_bag_add (property_bag,
				 "Name",
				 PROP_NAME,
				 BONOBO_ARG_STRING,
				 NULL,
				 "The name of the project",
				 0);
	bonobo_property_bag_add (property_bag,
				 "Company",
				 PROP_COMPANY,
				 BONOBO_ARG_STRING,
				 NULL,
				 "The company that drives the project",
				 0);
	bonobo_property_bag_add (property_bag,
				 "Manager",
				 PROP_MANAGER,
				 BONOBO_ARG_STRING,
				 NULL,
				 "The project manager",
				 0);

	priv->property_bag = property_bag;

	event_source = bonobo_event_source_new ();
	project->event_source = event_source;
	bonobo_object_add_interface (BONOBO_OBJECT (project),
				     BONOBO_OBJECT (event_source));
}
